@@ -22,6 +22,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
+from pyrap.tables  import table
 
 from DDFacet.compatibility import range
 
@@ -39,7 +40,6 @@ import sys,os
 if "PYTHONPATH_FIRST" in os.environ.keys() and int(os.environ["PYTHONPATH_FIRST"]):
     sys.path = os.environ["PYTHONPATH"].split(":") + sys.path
 
-
 #import matplotlib
 # matplotlib.use('agg')
 import optparse
@@ -102,6 +102,11 @@ import DDFacet
 print("DDFacet version is",report_version())
 print("Using python package located at: " + os.path.dirname(DDFacet.__file__))
 print("Using driver file located at: " + __file__)
+
+# hack to avoid recursion depth issues in SSD
+
+sys.setrecursionlimit(10000)
+
 global Parset
 Parset = ReadCFG.Parset("%s/DefaultParset.cfg" % os.path.dirname(DDFacet.Parset.__file__))
 
@@ -162,7 +167,8 @@ def main(OP=None, messages=[]):
     logger.logToFile(ImageName + ".log", append=DicoConfig["Log"]["Append"])
     global log
     log = logger.getLogger("DDFacet")
-
+    logger.setSilent("app")
+    
     # disable colors and progressbars if requested
     ModColor.silent = SkyModel.Other.ModColor.silent = \
                       progressbar.ProgressBar.silent = \
@@ -279,6 +285,7 @@ def main(OP=None, messages=[]):
                                                   deconvolve=("Clean" in Mode))
 
     Imager.Init()
+        
 
     # Imager.testDegrid()
     # stop
@@ -470,7 +477,7 @@ if __name__ == "__main__":
         print(ModColor.Str("  the original underlying error may be reported in the log [possibly far] above."), file=log)
         report_error = True
     except:
-        if sys.exc_info()[0] is not WorkerProcessError and Exceptions.is_pdb_enabled():
+        if sys.exc_info()[0]!=WorkerProcessError and Exceptions.is_pdb_enabled():
             APP.terminate()
             raise
         else:
