@@ -34,9 +34,32 @@ from DDFacet.Array import ModLinAlg
 from DDFacet.Other.progressbar import ProgressBar
 from DDFacet.Data import ClassLOFARBeam
 from DDFacet.Data import ClassFITSBeam
+from DDFacet.Data import ClassGMRTBeam
+from DDFacet.Data import ClassATCABeam as ClassATCABeam
+
 # import ClassSmoothJones is not used anywhere, should be able to remove it
+
 import tables
+import glob
 from scipy.interpolate import interp1d
+import casacore.tables as pt
+import hashlib
+import DDFacet.Other.PrintList
+
+def _which_solsfile(sol_files, req_times, solset, apply_map):
+    central_time = np.mean(np.unique(req_times))
+    sol_times = []
+    for sol_file in sol_files:
+        with tables.open_file(sol_file) as H:
+            _solset = getattr(H.root, solset)
+            for soltab, v in apply_map.items():
+                if not v:
+                    continue
+                _soltab = getattr(_solset, soltab)
+                sol_times.append(np.mean(_soltab.time[:]))
+                break
+    closest = np.argmin(np.abs(np.subtract(central_time, np.array(sol_times))))
+    return sol_files[closest]
 
 def _parse_solsfile(SolsFile):
     """Parses the input SolsFile in order to use an input h5parm file to apply solutions on-the-fly.
@@ -48,7 +71,7 @@ def _parse_solsfile(SolsFile):
 
             Everything before the `:` is the h5parm path.
             The h5parm file must follow the format defined by losoto>=2.0
-            The axis ordering for all soltabs is:
+            The axis ordering for all soltabs is internally converted to:
                 [pol, dir, ant, freq, time] for soltabs with frequency dependence
                 [pol, dir, ant, time] for soltabs without frequency dependence
 
@@ -113,7 +136,7 @@ def _parse_solsfile(SolsFile):
         raise ValueError('No solsets provided')
     if apply_map['tec000'] and apply_map['phase000']:
         raise ValueError("Cannot apply both phase and tec")
-    if ~np.any(np.array(apply_map.values())):
+    if ~np.any(np.array(list(apply_map.values()))):
         raise ValueError("No valid soltabs specified")
     return h5file, apply_solsets, apply_map
 
@@ -153,17 +176,53 @@ class ClassJones():
                 DicoSols, TimeMapping, DicoClusterDirs = self.MakeSols("killMS", DATA, quiet=quiet)
                 if self.CacheMode: self.MS.cache.saveCache("JonesNorm_killMS")
 
+            # DEBUG plot
+            #if True:
+            #    print('Debugging plots...')
+            #    import pylab as plt
+            #    import os
+            #    output = os.path.abspath('./debug_figs2')
+            #    if not os.path.exists(output):
+            #        os.makedirs(output)
+            #    Nt, Nd, Na, Nf, _, _ = DicoSols['Jones'].shape
+            #    eff_phase = np.angle(DicoSols['Jones'])
+            #    print('len time mapping:', len(TimeMapping))
+            #    for d in range(Nd):
+            #        print('d %i' % d)
+            #        for a in range(Na):
+            #            print('a %i' % a)
+            #            fig, axs = plt.subplots(1,1,figsize=(20,20))
+            #            #plt.colorbar(img)
+            #            img=axs.imshow(eff_phase[TimeMapping,d,a,:,0,0].T, vmin = - np.pi, vmax = np.pi,aspect='auto', cmap = plt.cm.jet)
+            #            print(eff_phase[TimeMapping,d,a,:,0,0].T.shape)
+            #    
+            #            #plt.colorbar(img)
+            #            plt.savefig(os.path.join(output,'gains_{}_{}.png'.format(d,a)))
+            #            plt.close('all')
+            #    exit()
+
             DATA["killMS"] =  dict(Jones=DicoSols, TimeMapping=TimeMapping, Dirs=DicoClusterDirs)
             self.DicoClusterDirs_kMS=DicoClusterDirs
 
             self.HasKillMSSols = True
 
-
-        ApplyBeam=(GD["Beam"]["Model"] is not None) and (GD["Beam"]["Model"] is not "")
+        ApplyBeam=(GD["Beam"]["Model"] is not None) and (GD["Beam"]["Model"]!="")
         if ApplyBeam:
             self.ApplyCal = True
             valid=False
             if self.CacheMode:
+                def __hashjsonbeamsets(GD):
+                    beamsets = GD["Beam"]["FITSFile"]
+                    if not isinstance(beamsets, list):
+                        beamsets = beamsets.split(',')
+                    lbeamsethashes = {}
+                    for bs in beamsets:
+                        if os.path.splitext(bs)[1] == ".json":
+                            if os.path.exists(bs):
+                                with open(bs, "r") as fbs:
+                                    lbeamsethashes["bs"] = hashlib.md5(fbs.read().encode()).hexdigest()
+                    return lbeamsethashes
+
                 self.JonesNormSolsFile_Beam, valid = self.MS.cache.checkCache("JonesNorm_Beam.npz", 
                                                                               dict(VisData=GD["Data"], 
                                                                                    Beam=GD["Beam"], 
@@ -232,7 +291,6 @@ class ClassJones():
         Jones=np.load("%s.npy"%InName)
         print("  %s.npy loaded" % InName, file=log)
 
-
         DicoClusterDirs = {}
         DicoClusterDirs["l"] = SolsFile["l"]
         DicoClusterDirs["m"] = SolsFile["m"]
@@ -366,8 +424,8 @@ class ClassJones():
         print("  Build Time Mapping", file=log)
         DicoJonesMatrices = DicoSols
         ind = np.zeros((times.size,), np.int32)
-        nt, na, nd, _, _, _ = DicoJonesMatrices["Jones"].shape
-        ii = 0
+        nt, _, _, _, _, _ = DicoJonesMatrices["Jones"].shape
+        #ii = 0
         for it in range(nt):
             t0 = DicoJonesMatrices["t0"][it]
             t1 = DicoJonesMatrices["t1"][it]
@@ -417,7 +475,8 @@ class ClassJones():
             
             DicoClusterDirs, DicoSols, VisToJonesChanMapping = self.GiveKillMSSols_SingleFile(
                 File, GlobalMode=ThisGlobalMode, JonesMode=ThisJonesMode)
-            print("  VisToJonesChanMapping: %s" % str(VisToJonesChanMapping), file=log)
+
+            print("  VisToJonesChanMapping: %s" % DDFacet.Other.PrintList.ListToStr(VisToJonesChanMapping), file=log)
             ListDicoSols.append(DicoSols)
             #if isol==1: stop
             #isol+=1
@@ -430,7 +489,7 @@ class ClassJones():
         for DicoJones1 in ListDicoSols[1::]:
             DicoJones = self.MergeJones(DicoJones1, DicoJones)
             VisToJonesChanMapping = self.GiveVisToJonesChanMapping(DicoJones["FreqDomains"])
-            print("  VisToJonesChanMapping: %s" % str(VisToJonesChanMapping), file=log)
+            print("  VisToJonesChanMapping: %s" % DDFacet.Other.PrintList.ListToStr(VisToJonesChanMapping), file=log)
         #stop
         DicoJones["VisToJonesChanMapping"] = VisToJonesChanMapping
 
@@ -528,70 +587,126 @@ class ClassJones():
             If nothing comes after the `:` or there is no `:` then we assume the
             solution spec of `sol000/tec000`.
         """
+        # TODO: if more than one solset present for a MS file and dirs or times differ, this will fail.
+
+        def reorderAxes( a, oldAxes, newAxes ):
+            """
+            Reorder axis of an array to match a new name pattern.
+        
+            Parameters
+            ----------
+            a : np array
+                The array to transpose.
+            oldAxes : list of str
+                A list like ['time','freq','pol'].
+                It can contain more axes than the new list, those are ignored.
+                This is to pass to oldAxis the soltab.getAxesNames() directly even on an array from getValuesIter()
+            newAxes : list of str
+                A list like ['time','pol','freq'].
+        
+            Returns
+            -------
+            np array
+                With axis transposed to match the newAxes list.
+            """
+            oldAxes = [ax for ax in oldAxes if ax in newAxes]
+            idx = [ oldAxes.index(ax) for ax in newAxes ]
+            return np.transpose(a, idx)
 
         self.ApplyCal = True
 
         h5file, apply_solsets, apply_map = _parse_solsfile(SolsFile)
-        print("  Applying {} solset {} soltabs {}".format(h5file, apply_solsets, apply_map), file=log)
+        print("Parsing h5file pattern {}".format(h5file), file=log)
+        h5files = glob.glob(h5file)
+        with pt.table(self.MS.MSName, ack=False) as t:
+            req_times = np.unique(t.getcol('TIME'))
+        with pt.table(self.MS.MSName+'/ANTENNA', ack=False) as t:
+            req_ants = t.getcol('NAME')
+        h5file = _which_solsfile(h5files, req_times, apply_solsets[0], apply_map)
+        print( "  Applying {} solset {} soltabs {}".format(h5file, apply_solsets, apply_map), file=log)
 
-        times = None
+        # times = None
         with tables.open_file(h5file) as H:
-            gains = []
+            # Prepare unite Jones matrix array to be filled with the gains
+            first_soltab = [key for key, value in apply_map.items() if value][0] # get name of the first soltab
+            # Restrict Jones matrices to times present in the MS file.
+            t_h5 = getattr(getattr(H.root,apply_solsets[0]), first_soltab).time # times in h5parm
+            t_h5_startidx = np.argmin(np.abs(t_h5 - req_times[0])) # time idx in h5 closest to first time in MS
+            t_h5_stopidx = np.argmin(np.abs(t_h5 - req_times[-1])) + 1 # time idx in h5 after closest to end time in MS
+            times = t_h5[t_h5_startidx:t_h5_stopidx]
+            Nt = len(times)
+            Nd = len(getattr(H.root,apply_solsets[0]).source[:]['name'])
+            Na = len(req_ants)
+            freqs = self.MS.ChanFreq.ravel().astype(np.float32)
+            Nf = freqs.size
+            Np = 2
+            gains = np.ones((Nt,Nd,Na,Nf,Np), dtype=np.complex64)
+
             lm, radec = [], []
             for solset in apply_solsets:
                 _solset = getattr(H.root, solset)
+
+                dirnames_solset = _solset.source[:]['name'] # keep track to re-arrange order of solutions later
                 raNode, decNode = _solset.source[:]["dir"].T
                 lFacet, mFacet = self.FacetMachine.CoordMachine.radec2lm(raNode, decNode)
                 radec.append(np.stack([raNode, decNode], axis=1))
                 lm.append(np.stack([lFacet, mFacet], axis=1))
-                # freqs=self.FacetMachine.VS.GlobalFreqs.reshape((1,1,1,-1))
-                freqs = self.MS.ChanFreq.ravel()
-                Nf = freqs.size
-                tec_conv = -8.4479745e6 / freqs
 
-                solset_gains = []
                 for soltab, v in apply_map.items():
                     if not v:
                         continue
                     _soltab = getattr(_solset, soltab)
-                    if times is not None:
-                        if ~np.all(np.isclose(_soltab.time[:], times)):
-                            raise ValueError("Times not the same between solsets")
-                    times = _soltab.time[:]
-                    # Npols, Nd, Na, (Nf), Nt
-                    val = _soltab.val[:]
+                    # assert times
+                    if not len(_soltab.time) >= t_h5_stopidx:
+                        raise ValueError("Times not the same between solsets")
+                    if ~np.all(np.isclose(_soltab.time[t_h5_startidx:t_h5_stopidx], times)):
+                        raise ValueError("Times not the same between solsets")
+
+                    val = np.array(_soltab.val, np.float32) # Npols, Nd, Na, (Nf), Nt - arbitrary order
+                    val[_soltab.weight == 0] = np.nan # set flagged data to nan
+                    # check axes order and reshape
+                    axes_order = _soltab.val.attrs['AXES'].decode().split(',')
+                    if 'freq' in axes_order: # phase, amplitude
+                        val = reorderAxes(val, axes_order, ['dir', 'ant', 'freq', 'time', 'pol'])
+                    else: # tec
+                        val = reorderAxes(val, axes_order, ['dir', 'ant', 'time', 'pol'])
+                    val = val[...,t_h5_startidx:t_h5_stopidx,:] # select only time range also in MS
+                    # times = _soltab.time[:]
+
+                    antnames_soltab = _soltab.ant[:].tolist()
+                    ant_idx = [antnames_soltab.index(name.encode()) for name in req_ants] # find only antennas useful for this dataset
+                    try:
+                        dirnames_soltab = _soltab.dir[:].tolist()
+                    except:
+                        dirnames_soltab = _soltab.dir[:] # to be removed when no old soltab are around
+                    dir_idx = [dirnames_soltab.index(name) for name in dirnames_solset] # find only directions useful for this dataset
                     if soltab == 'tec000':
-                        _, Nd, Na, Nt = val.shape
-                        # Nd, Na, Nt, Nf
-                        phase = tec_conv * val[0, ..., None]
-                        # Nt,Nd,Na,Nf
-                        phase = phase.transpose((2, 0, 1, 3))
-                        solset_gains.append(np.exp(1j * phase))
+                        tec_conv = (-8.4479745e6 / freqs).astype(np.float32)
+                        val = val[dir_idx][:,ant_idx]
+                        # Nd, Na, Nt, Np, Nf
+                        phase = tec_conv * val[..., None]
+                        # Nt, Nd, Na, Nf, Np
+                        phase = phase.transpose((2, 0, 1, 4, 3))
+                        gains *= np.exp(1j * phase)
                     if soltab == 'phase000':
-                        _, Nd, Na, _, Nt = val.shape
-                        _freqs = _soltab.freq[:]
-                        # Nd, Na, Nf, Nt
-                        phase = interp1d(_freqs, val[0, ...], axis=-2, kind='nearest', bounds_error=False,
-                                         fill_value='extrapolate')(freqs)
-                        # Nt,Nd,Na,Nf
-                        phase = phase.transpose((3, 0, 1, 2))
-                        solset_gains.append(np.exp(1j * phase))
+                        val = val[dir_idx][:,ant_idx]
+                        _freqs = np.array(_soltab.freq, np.float32)
+                        # Nd, Na, Nf, Nt, Np
+                        phase = interp1d(_freqs, val, axis=2, kind='nearest', bounds_error=False,
+                                    fill_value='extrapolate')(freqs) # linear?
+                        # Nt, Nd, Na, Nf, Np
+                        phase = phase.transpose((3, 0, 1, 2, 4))
+                        gains *= np.exp(1j * phase)
                     if soltab == 'amplitude000':
-                        _, Nd, Na, _, Nt = val.shape
-                        _freqs = _soltab.freq[:]
-                        # Nd, Na, Nf, Nt
-                        amplitude = np.abs(
-                            interp1d(_freqs, val[0, ...], axis=-2, kind='nearest', bounds_error=False,
-                                     fill_value='extrapolate')(freqs))
-                        amplitude = np.maximum(amplitude, 0.01)
-                        # Nt,Nd,Na,Nf
-                        amplitude = amplitude.transpose((3, 0, 1, 2))
-                        solset_gains.append(amplitude)
-                # Nt,Nd,Na,Nf
-                gains.append(np.prod(solset_gains, axis=0))
-            # Nt, (Nd+Nd+...), Na, Nf
-            gains = np.concatenate(gains, axis=1)
-            # Nd+Nd+...
+                        val = val[dir_idx][:,ant_idx]
+                        _freqs = np.array(_soltab.freq, np.float32)
+                        # Nd, Na, Nf, Nt, Np
+                        amplitude = np.abs(interp1d(_freqs, val, axis=2, kind='nearest', bounds_error=False,
+                                             fill_value='extrapolate')(freqs)) # linear?
+                        # Nt, Nd, Na, Nf, Np
+                        amplitude = amplitude.transpose((3, 0, 1, 2, 4))
+                        gains *= amplitude
+            # Nd,2
             lm = np.concatenate(lm, axis=0)
             radec = np.concatenate(radec, axis=0)
 
@@ -616,45 +731,45 @@ class ClassJones():
         ClusterCat.Cluster = DicoClusterDirs["Cluster"]
         self.ClusterCat = ClusterCat
 
-        dts = np.diff(times)
-        dt = np.median(dts)
-        # undo killms2h5parm times = (t0+t1)/2.
-        t0 = times - dt / 2.
-        t1 = times + dt / 2.
+        # find time ranges of solutions
+        diff = np.diff(times)/2.
+        t_mid = times[1:] - diff # mid point per interval
         DicoSols = {}
-        DicoSols["t0"] = t0
-        DicoSols["t1"] = t1
+        DicoSols["t0"] = np.insert(t_mid, 0, times[0]-diff[0])
+        DicoSols["t1"] = np.append(t_mid, times[-1]+diff[-1])
         DicoSols["tm"] = times
 
-        Nt, Nd, Na, Nf = gains.shape
+        Nt, Nd, Na, Nf, Np = gains.shape
 
         G = np.zeros((Nt, Nd, Na, Nf, 2, 2), np.complex64)
 
-        G[:, :, :, :, 0, 0] = gains
-        G[:, :, :, :, 1, 1] = gains
+        gains[np.isnan(gains)] = 1.
+        G[:, :, :, :, 0, 0] = gains[...,0] # XX
+        G[:, :, :, :, 1, 1] = gains[...,1] # YY
 
-        #        import pylab as plt
-        #        import os
-        #        output = os.path.abspath('./debug_figs')
-        #        if not os.path.exists(output):
-        #            os.makedirs(output)
-        #        eff_phase = np.angle(G)
-        #        eff_amp = np.abs(G)
-        #        print(freqs, _freqs)
-        #        for d in range(Nd):
-        #            for a in range(Na):
-        #                fig, axs = plt.subplots(4,1,figsize=(20,20))
-        #                img = axs[0].imshow(eff_amp[:,d,a,:,0,0].T, cmap='hsv', vmin = 0.8, vmax = 1.2,aspect='auto')
-        ##                plt.colorbar(img)
-        #                img=axs[1].imshow(eff_phase[:,d,a,:,0,0].T, cmap='hsv', vmin = - np.pi, vmax = np.pi,aspect='auto')
-        #                img = axs[2].plot(np.std(eff_amp[:,d,a,:,0,0],axis=1))
-        ##                plt.colorbar(img)
-        #                img=axs[3].plot(np.std(eff_phase[:,d,a,:,0,0],axis=1))
-        #
-        # #               plt.colorbar(img)
-        #                plt.savefig(os.path.join(output,'gains_{}_{}.png'.format(d,a)))
-        #                plt.close('all')
-        #        exit()
+        # DEBUG plot
+        #if True:
+        #    import pylab as plt
+        #    import os
+        #    output = os.path.abspath('./debug_figs')
+        #    if not os.path.exists(output):
+        #        os.makedirs(output)
+        #    eff_phase = np.angle(G)
+        #    eff_amp = np.abs(G)
+        #    #print(freqs, _freqs)
+        #    for d in range(Nd):
+        #        for a in range(23,Na):
+        #            fig, axs = plt.subplots(4,1,figsize=(20,20))
+        #            img = axs[0].imshow(eff_amp[:,d,a,:,0,0].T, cmap='hsv', vmin = 0.8, vmax = 1.2,aspect='auto')
+        #            #plt.colorbar(img)
+        #            img=axs[1].imshow(eff_phase[:,d,a,:,0,0].T, vmin = - np.pi, vmax = np.pi,aspect='auto', cmap = plt.cm.jet)
+        #            img = axs[2].plot(np.std(eff_amp[:,d,a,:,0,0],axis=1))
+        #            #plt.colorbar(img)
+        #            img=axs[3].plot(np.std(eff_phase[:,d,a,:,0,0],axis=1))
+        #    
+        #            #plt.colorbar(img)
+        #            plt.savefig(os.path.join(output,'gains_{}_{}.png'.format(d,a)))
+        #            plt.close('all')
 
         VisToJonesChanMapping = np.int32(np.arange(self.MS.NSPWChan, ))
 
@@ -663,15 +778,12 @@ class ClassJones():
         return VisToJonesChanMapping, DicoClusterDirs, DicoSols, G
 
 
-
-
     def GiveKillMSSols_SingleFile(
         self,
         SolsFile,
         JonesMode="AP",
         GlobalMode=""):
 
-
         if not ".h5" in SolsFile:
             # if not(".npz" in SolsFile):
             #     Method = SolsFile
@@ -789,9 +901,6 @@ class ClassJones():
                 G[:,iDir,:,:,1,1]=G[:,iDir,:,:,1,1]/gu
 
 
-
-
-
         if not("A" in JonesMode):
             print("  Normalising by the amplitude", file=log)
             G[G != 0.] /= np.abs(G[G != 0.])
@@ -851,15 +960,17 @@ class ClassJones():
             #print>>log, "  Estimating LOFAR beam model in %s mode every %5.1f min."%(LOFARBeamMode,DtBeamMin)
             # self.GiveInstrumentBeam=self.MS.GiveBeam
             # estimate beam sample times using DtBeamMin
-
         elif GD["Beam"]["Model"] == "FITS":
             self.BeamMachine = ClassFITSBeam.ClassFITSBeam(self.MS, GD["Beam"])
             self.GiveInstrumentBeam = self.BeamMachine.evaluateBeam
-
-            # self.DtBeamDeg = GD["Beam"]["FITSParAngleIncrement"]
-            # print>>log, "  Estimating FITS beam model every %5.1f min."%DtBeamMin
+        elif GD["Beam"]["Model"] == "GMRT":
+            self.BeamMachine = ClassGMRTBeam.ClassGMRTBeam(self.MS, GD["Beam"])
+            self.GiveInstrumentBeam = self.BeamMachine.GiveInstrumentBeam
+        elif GD["Beam"]["Model"] == "ATCA":
+            self.BeamMachine = ClassATCABeam.ClassATCABeam(self.MS,GD["Beam"])
+            self.GiveInstrumentBeam = self.BeamMachine.GiveInstrumentBeam            
         else:
-            raise ValueError("Unknown keyword for Beam-Model. Only accepts 'FITS' or 'LOFAR'")
+            raise ValueError("Unknown keyword for Beam-Model. Only accepts 'FITS', 'LOFAR', 'GMRT' or 'ATCA'")
 
     def GiveBeam(self, times, quiet=False,RaDec=None):
         GD = self.GD
@@ -914,7 +1025,7 @@ class ClassJones():
 
         DicoBeam["VisToJonesChanMapping"]=self.GiveVisToJonesChanMapping(FreqDomains)
         if not quiet:
-            print("VisToJonesChanMapping: %s"%DicoBeam["VisToJonesChanMapping"], file=log)
+            print("VisToJonesChanMapping: %s"%DDFacet.Other.PrintList.ListToStr(DicoBeam["VisToJonesChanMapping"]), file=log)
 
 
         DicoBeam["Jones"]=np.zeros((Tm.size,NDir,self.MS.na,FreqDomains.shape[0],2,2),dtype=np.complex64)
