@@ -27,7 +27,9 @@ from DDFacet.compatibility import range
 import os, re, glob
 import pyrap.measures as pm
 import pyrap.quanta as qa
+from astropy.time import Time
 from pyrap.tables import table
+from datetime import datetime as dt
 
 import ephem
 import numpy as np
@@ -59,7 +61,10 @@ from DDFacet.Other.progressbar import ProgressBar
 
 
 class ClassMS():
-    def __init__(self,MSname,Col="DATA",zero_flag=True,ReOrder=False,EqualizeFlag=False,DoPrint=True,DoReadData=True,
+    def __init__(self,MSname,
+                 Col="DATA",
+                 SubCol=None,
+                 zero_flag=True,ReOrder=False,EqualizeFlag=False,DoPrint=True,DoReadData=True,
                  TimeChunkSize=None,GetBeam=False,RejectAutoCorr=False,SelectSPW=None,DelStationList=None,
                  AverageTimeFreq=None,
                  Field=0,DDID=0,TaQL=None,ChanSlice=None,GD=None,
@@ -99,7 +104,10 @@ class ClassMS():
 
         self.AverageSteps=AverageTimeFreq
         self.MSName = MSName = reformat.reformat(os.path.abspath(MSname), LastSlash=False)
+        
         self.ColName=Col
+        self.SubColName=SubCol
+        
         self.ChanSlice = ChanSlice or slice(None)
         self.zero_flag=zero_flag
         self.ReOrder=ReOrder
@@ -121,7 +129,7 @@ class ClassMS():
         # once.
         self._reset_cache = ResetCache
         self._chunk_caches = {}
-        self.maincache = CacheManager(MSname+".F%d.D%d.ddfcache"%(self.Field, self.DDID), reset=ResetCache, cachedir=self.GD["Cache"]["Dir"], nfswarn=True)
+        self.maincache = CacheManager(MSName+".F%d.D%d.ddfcache"%(self.Field, self.DDID), reset=ResetCache, cachedir=self.GD["Cache"]["Dir"], nfswarn=True)
 
         self.ReadMSInfo(first_ms=first_ms,DoPrint=DoPrint)
         self.LFlaggedStations=[]
@@ -383,7 +391,7 @@ class ClassMS():
             row1=self.nbl
         A0=self.F_A0[row0:row1]
         A1=self.F_A1[row0:row1]
-        MapOut=np.ones((self.nbl,),dtype=np.bool)
+        MapOut=np.ones((self.nbl,),dtype=bool)
         if FlagAutoCorr:
             ind=np.where(A0==A1)[0]
             MapOut[ind]=False
@@ -677,13 +685,29 @@ class ClassMS():
                 if sort_index is not None:
                     visdata1 = np.ndarray(shape=datashape, dtype=np.complex64)
                     table_all.getcolslicenp(self.ColName, visdata1, self.cs_tlc, self.cs_brc, self.cs_inc, row0, nRowRead)
+                    if self.SubColName is not None:
+                        for SubCol in self.SubColName:
+                            print("  substracting MS visibilities from column %s" % SubCol, file=log)
+                            visdata2 = np.ndarray(shape=datashape, dtype=np.complex64)
+                            table_all.getcolslicenp(SubCol, visdata2, self.cs_tlc, self.cs_brc, self.cs_inc, row0, nRowRead)
+                            visdata1-=visdata2
+                            del visdata2
                     print("sorting visibilities", file=log)
                     t0 = time.time()
                     visdata[...] = visdata1[sort_index]
                     del visdata1
                     print("sorting took %.1fs"%(time.time()-t0), file=log)
                 else:
+                    #print(self.ColName, visdata.shape, visdata.dtype, self.cs_tlc, self.cs_brc, self.cs_inc, row0, nRowRead)
                     table_all.getcolslicenp(self.ColName, visdata, self.cs_tlc, self.cs_brc, self.cs_inc, row0, nRowRead)
+                    if self.SubColName is not None:
+                        for SubCol in self.SubColName:
+                            print("  substracting MS visibilities from column %s" % SubCol, file=log)
+                            visdata2 = np.ndarray(shape=datashape, dtype=np.complex64)
+                            table_all.getcolslicenp(SubCol, visdata2, self.cs_tlc, self.cs_brc, self.cs_inc, row0, nRowRead)
+                            visdata-=visdata2
+                            del visdata2
+                    #visdata[...]=table_all.getcolslice(self.ColName, self.cs_tlc, self.cs_brc, self.cs_inc, row0, nRowRead)
                 if self._reverse_channel_order:
                     visdata[:,:,:]= visdata[:,::-1,:]
   
@@ -973,7 +997,8 @@ class ClassMS():
         tp = table(table_all.getkeyword('POLARIZATION'),ack=False)
         # get list of corrype enums for first row of polarization table, and convert to strings via MS_STOKES_ENUMS. 
         # self.CorrelationNames will be a list of strings
-        self.CorrelationIds = tp.getcol('CORR_TYPE',0,1)[self._polid]
+        self.CorrelationIds = tp.getcol('CORR_TYPE',self._polid,1)[0]
+
         self.CorrelationNames = [ (ctype >= 0 and ctype < len(MS_STOKES_ENUMS) and MS_STOKES_ENUMS[ctype]) or
                 None for ctype in self.CorrelationIds ]
         self.Ncorr = len(self.CorrelationNames)
@@ -1088,6 +1113,17 @@ class ClassMS():
 
         ta=table(table_all.getkeyword('FIELD'),ack=False)
         rarad,decrad=ta.getcol('PHASE_DIR')[self.Field][0]
+        if np.abs(decrad)>=np.pi/2:
+            log.print(ModColor.Str("BE CAREFUL SOME SOFTWARE HAVE BEEN SHOWN TO NOT PROPERLY MANAGE DEC=90 DEGREES"))
+            log.print(ModColor.Str("   ... will slightly move the assumed phase center..."))
+            U,V,W=table_all.getcol("UVW").T
+            d=np.max(np.sqrt(U**2+V**2))
+            wavelmin=3e8/self.ChanFreq.max()
+            ddecrad=(wavelmin/d)/100
+            decrad-=ddecrad
+            ddec_arcsec=ddecrad*180/np.pi*3600
+            log.print(ModColor.Str("   ... moving phase center by 1/100th of a resolution element... (%.3f arcsec))"%ddec_arcsec))
+            
         if rarad<0.: rarad+=2.*np.pi
         self.OriginalRadec = self.OldRadec = rarad,decrad
 
@@ -1097,6 +1133,11 @@ class ClassMS():
             if self.ToRADEC == "align":
                 if first_ms is not None:
                     ranew, decnew = first_ms.rarad, first_ms.decrad
+                    # if (ranew, decnew)!=self.OriginalRadec:
+                    #     self.GD["Image"]["PhaseCenterRADEC"]=[rad2hmsdms(first_ms.rarad,Type="ra").replace(" ",":"),rad2hmsdms(first_ms.decrad,Type="dec").replace(" ",":")]
+                    #     log.print("PhaseCenterRADEC in 'align'")
+                    #     log.print("   set the PhaseCenterRADEC in the parset to be: %s"%str(self.GD["Image"]["PhaseCenterRADEC"]))
+
                 which = "the common phase centre"
             else:
                 which = "%s %s"%tuple(self.ToRADEC)
@@ -1205,19 +1246,21 @@ class ClassMS():
 
         if self.DicoSelectOptions["UVRangeKm"]:
             d0, d1 = self.DicoSelectOptions["UVRangeKm"]
-            print("  flagging uv data outside uv distance of [%5.1f~%5.1f] km" % (d0, d1), file=log)
+            print("  flagging uv data outside uv distance of [%5.2f~%5.2f] km" % (d0, d1), file=log)
             d0 = d0**2*1e6
             d1 = d1**2*1e6
             duv = (uvw[:,:2]**2).sum(1)  # u^2+v^2... and we already squared d0 and d1
             flags[(duv < d0) | (duv > d1),:,:] = True
 
         if self.DicoSelectOptions["TimeRange"]:
-            t0 = times[0]
-            tt = (times - t0) / 3600.
-            st0, st1 = self.DicoSelectOptions["TimeRange"]
-            print("  selecting uv data in time range [%.4f~%5.4f] hours" % (st0, st1), file=log)
-            ind = np.where((tt >= st0) & (tt < st1))[0]
-            flags[ind, :, :] = True
+            st0, st1 = list(map(lambda x: dt.utcfromtimestamp(qa.quantity(x).to_unix_time()),
+                                self.DicoSelectOptions["TimeRange"]))
+            print("  imaging only uv data in time range [{}~{}] UTC".format(st0, st1), file=log)
+            st0, st1 = list(map(lambda x: qa.quantity(x).to_unix_time(),
+                                self.DicoSelectOptions["TimeRange"]))
+            times_utc = np.array(list(map(lambda x: qa.quantity("{}s".format(x)).to_unix_time(), times)))
+            sel = np.logical_or(times_utc < st0, times_utc > st1)
+            flags[sel, :, :] = True
 
         if self.DicoSelectOptions["DistMaxToCore"]:
             DMax = self.DicoSelectOptions["DistMaxToCore"] * 1e3
@@ -1245,9 +1288,9 @@ class ClassMS():
         antenna_flagfrac = [flags1[rows].sum() / float(flags1[rows].size or 1) for rows in antenna_rows]
         print("  flagged fractions per antenna: %s" % " ".join(["%.2f" % frac for frac in antenna_flagfrac]), file=log)
 
+        
         FlagAntFrac = [ant for ant, frac in enumerate(antenna_flagfrac) if frac > ThresholdFlag]
         FlagAntNumber.update(FlagAntFrac)
-
         for A in FlagAntFrac:
             print("    antenna %i has ~%4.1f%s of flagged data (more than %4.1f%s)" % \
                          (A, antenna_flagfrac[A] * 100, "%", ThresholdFlag * 100, "%"), file=log)
@@ -1277,9 +1320,10 @@ class ClassMS():
         ll.append("   - Phase centre (field %d): (ra, dec)=(%s, %s) "%(self.Field, rad2hmsdms(rarad,Type="ra").replace(" ",":")\
                                                                        ,rad2hmsdms(decrad,Type="dec").replace(" ",".")))
         ll.append("   - Frequency = %s MHz"%str(np.mean(self.ChanFreq)/1e6))
-        ll.append("   - Wavelength = %5.2f meters"%(np.mean(self.wavelength_chan)))
+        #ll.append("   - Wavelength = %5.2f meters"%(np.mean(self.wavelength_chan)))
         Freqs=3.e8/self.wavelength_chan.ravel()/1e6
-        ll.append("   - Channel frequencies = %s MHz"%(str((self.ChanFreq/1e6).tolist())))
+        s=" ".join(["%.2f"%x for x in Freqs])
+        ll.append("   - Channel frequencies = %s"%(DDFacet.Other.PrintList.ListToStr(s.split(" "),Unit="MHz")))
         ll.append("   - Bandwidth = %5.2f MHz"%(np.max(Freqs)-np.min(Freqs)))
         ll.append("   - Time bin = %4.1f seconds"%(self.dt))
         ll.append("   - Total Integration time = %6.2f hours"%self.DTh)
@@ -1302,11 +1346,15 @@ class ClassMS():
         return l,m
 
 
-    def PutVisColumn(self, colname, vis, row0, row1, likecol="DATA", sort_index=None):
-        self.AddCol(colname, LikeCol=likecol, quiet=True)
+    def PutVisColumn(self, colname, vis, row0, row1, likecol="DATA", sort_index=None, ColDesc=None):
+        self.AddCol(colname,
+                    LikeCol=likecol,
+                    quiet=True,
+                    ColDesc=ColDesc)
+        
         nrow = row1 - row0
         if self._reverse_channel_order:
-            vis = vis[:,::-1,:]
+            vis = vis[:,::-1,...]
         print("writing column %s rows %d:%d"%(colname,row0,row1), file=log)
         t = self.GiveMainTable(readonly=False, ack=False)
 
@@ -1324,17 +1372,35 @@ class ClassMS():
                 vis0 = t.getcol(colname, row0, nrow)
             except RuntimeError:
                 vis0 = t.getcol("DATA", row0, nrow)
-            vis0[:, self.ChanSlice, :] = vis[reverse_index, :, :]
+            vis0[:, self.ChanSlice, ...] = vis[reverse_index, :, ...]
+            IsNan=np.isnan(vis0)
+            if np.count_nonzero(IsNan)>0:
+                log.print(ModColor.Str("There are NaNs in the array to be put in %s, replacing by zeros..."%colname))
+                vis0[IsNan]=0
             t.putcol(colname, vis0, row0, nrow)
         else:
             if sort_index is None:
                 vis0 = vis
             else:
-                vis0 = np.zeros((nrow,vis.shape[1],vis.shape[2]),vis.dtype)
+                if len(vis.shape)==3:
+                    vis0 = np.zeros((nrow,vis.shape[1],vis.shape[2]),vis.dtype)
+                elif len(vis.shape)==2:
+                    import warnings
+                    warnings.warn("Your dataset does not conform to NRAO MS v2 specification (memo 229) and only "
+                                  "contains 2D axis. We will assume this means nrow x nchan x (ncorr == 1). "
+                                  "Please notify your observatory of this issue.")
+                    vis0 = np.zeros((nrow,vis.shape[1]),vis.dtype)
                 vis0[sort_index,...] = vis
+
+            IsNan=np.isnan(vis0)
+            if np.count_nonzero(IsNan)>0:
+                log.print(ModColor.Str("  There are NaNs in the array to be put in %s, replacing by zeros..."%colname))
+                vis0[IsNan]=0
             t.putcol(colname, vis0, row0, nrow)
         t.close()
 
+
+        
     def SaveVis(self,vis=None,Col="CORRECTED_DATA",spw=0,DoPrint=True):
         if vis is None:
             vis=self.data
@@ -1462,7 +1528,7 @@ class ClassMS():
                 t.putcol(Colout,t.getcol(Colin,row0,NRow),row0,NRow)
         t.close()
 
-    def AddCol(self,ColName,LikeCol="DATA",quiet=False):
+    def AddCol(self,ColName,LikeCol="DATA",quiet=False,ColDesc=None):
         t=table(self.MSName,readonly=False,ack=False)
         if (ColName in t.colnames()):# and not self.GD["Predict"]["Overwrite"]):
             if not quiet:
@@ -1471,11 +1537,16 @@ class ClassMS():
             return
         # elif (ColName in t.colnames() and self.GD["Predict"]["Overwrite"]):
         #     t.removecols(ColName)
+        if ColDesc is None:
+            if "-" in LikeCol:
+                LikeCol=LikeCol.split("-")[0]
 
-        print("  Putting column %s in %s"%(ColName,self.MSName), file=log)
-        desc=t.getcoldesc(LikeCol)
+            desc=t.getcoldesc(LikeCol)
+        else:
+            desc=ColDesc
         desc["name"]=ColName
         desc['comment']=desc['comment'].replace(" ","_")
+        print("  Putting column %s in %s"%(ColName,self.MSName), file=log)
         t.addcols(desc)
         t.close()
         
@@ -1664,9 +1735,9 @@ def expandMSList(MSName,defaultField=0,defaultDDID=0,defaultColumn="DATA"):
     Ultimately, returns a list of (MSName, ddid, field) tuples, where MSName is a proper MS path, and ddid
     and field are indices.
     """
-    if type(MSName) is list:
+    if isinstance(MSName,list):
         print("multi-MS mode", file=log)
-    elif type(MSName) is not str:
+    elif not isinstance(MSName,str):
         raise TypeError("MSName parameter must be a list or a filename")
     elif MSName.endswith(".txt"):
         MSName0 = MSName
