@@ -321,6 +321,7 @@ class ClassDDEGridMachine():
                  cf_dict=None, compute_cf=False,
                  wmax=None,   # must be supplied if compute_cf=True
                  bda_grid=None, bda_degrid=None,
+                 Padding=None,
                  ):
         """
 
@@ -366,9 +367,10 @@ class ClassDDEGridMachine():
         elif Precision == "D":
             self.dtype = np.complex128
 
-        self.dtype = np.complex64
         T.timeit("0")
-        Padding = GD["Facets"]["Padding"]
+        if Padding is None:
+            Padding = GD["Facets"]["Padding"]
+        
         self.NonPaddedNpix, Npix = EstimateNpix(Npix, Padding)
         self.Padding = Npix/float(self.NonPaddedNpix)
         # self.Padding=Padding
@@ -784,32 +786,37 @@ class ClassDDEGridMachine():
                 ChanEquidistant,
                 self.SkyType,
                 self.PolModeID]
-            _pyGridderSmear.pyGridderWPol(Grid,
-                                          vis,
-                                          uvw,
-                                          flag,
-                                          W,
-                                          SumWeigths,
-                                          DoPSF,
-                                          self.WTerm.Wplanes,
-                                          self.WTerm.WplanesConj,
-                                          np.array([self.WTerm.RefWave,
-                                                    self.WTerm.wmax,
-                                                    len(self.WTerm.Wplanes),
-                                                    self.WTerm.OverS],
-                                                   dtype=np.float64),
-                                          self.incr.astype(np.float64),
-                                          freqs,
-                                          [self.PolMap,
-                                              FacetInfos],
-                                          ParamJonesList,
-                                          self._bda_grid,
-                                          sparsification if sparsification is not None else np.array([], dtype=np.bool),
-                                          OptimisationInfos,
-                                          self.LSmear,
-                                          np.int32(ChanMapping),
-                                          np.array(self.DataCorrelationFormat).astype(np.uint16),
-                                          np.array(self.ExpectedOutputStokes).astype(np.uint16))
+            if self.dtype == np.complex64:
+                gridfun = _pyGridderSmear.pyGridderWPol32
+            else:
+                gridfun = _pyGridderSmear.pyGridderWPol64
+            gridfun(Grid,
+                    vis,
+                    uvw,
+                    flag,
+                    W,
+                    SumWeigths,
+                    DoPSF,
+                    self.WTerm.Wplanes,
+                    self.WTerm.WplanesConj,
+                    np.array([self.WTerm.RefWave,
+                            self.WTerm.wmax,
+                            len(self.WTerm.Wplanes),
+                            self.WTerm.OverS],
+                            dtype=np.float64),
+                    self.incr.astype(np.float64),
+                    freqs,
+                    [self.PolMap,
+                        FacetInfos],
+                    ParamJonesList,
+                    self._bda_grid,
+                    sparsification if sparsification is not None else np.array([], dtype=np.bool),
+                    OptimisationInfos,
+                    self.LSmear,
+                    np.int32(ChanMapping),
+                    np.array(self.DataCorrelationFormat).astype(np.uint16),
+                    np.array(self.ExpectedOutputStokes).astype(np.uint16))
+        #endif BDA-grid
 
             T.timeit("gridder")
             T.timeit("grid %d" % self.IDFacet)
@@ -819,6 +826,8 @@ class ClassDDEGridMachine():
                 ChanEquidistant,
                 self.SkyType,
                 self.PolModeID]
+            if self.dtype != np.complex64:
+                raise RuntimeError("Classic BDA does not support double accumulation gridding")
             _pyGridderSmearClassic.pyGridderWPol(Grid,
                                           vis,
                                           uvw,
@@ -839,7 +848,7 @@ class ClassDDEGridMachine():
                                               FacetInfos],
                                           ParamJonesList,
                                           self._bda_grid,
-                                          sparsification if sparsification is not None else np.array([], dtype=np.bool),
+                                          sparsification if sparsification is not None else np.array([], dtype=bool),
                                           OptimisationInfos,
                                           self.LSmear,
                                           np.int32(ChanMapping),
@@ -863,7 +872,7 @@ class ClassDDEGridMachine():
         A1=None,
         Jones=None):
         if not isinstance(Grid, type(None)):
-            if not(Grid.dtype == np.complex64):
+            if not(Grid.dtype == self.dtype):
                 raise NameError(
                     'Grid.dtype %s %s' %
                     (str(
@@ -1092,7 +1101,7 @@ class ClassDDEGridMachine():
                 [self.PolMap, FacetInfos, RowInfos],
                 ParamJonesList, 
                 self._bda_degrid,
-                sparsification if sparsification is not None else np.array([], dtype=np.bool),
+                sparsification if sparsification is not None else np.array([], dtype=bool),
                 OptimisationInfos,
                 self.LSmear, np.int32(ChanMapping),
                 np.array(self.DataCorrelationFormat).astype(np.uint16),
@@ -1126,7 +1135,7 @@ class ClassDDEGridMachine():
                 [self.PolMap, FacetInfos, RowInfos],
                 ParamJonesList, 
                 self._bda_degrid,
-                sparsification if sparsification is not None else np.array([], dtype=np.bool),
+                sparsification if sparsification is not None else np.array([], dtype=bool),
                 OptimisationInfos,
                 self.LSmear, np.int32(ChanMapping),
                 np.array(self.DataCorrelationFormat).astype(np.uint16),
