@@ -94,10 +94,13 @@ class ClassFacetMachine():
             self.stitchedType = np.float32  # cleaning requires float32
         elif Precision == "D":
             self.dtype = np.complex128
-            self.CType = np.complex64
+            self.CType = np.complex128
             self.FType = np.float64
             self.stitchedType = np.float32  # cleaning requires float32
 
+        if int(GD["CF"]["Nw"]) <= 1:
+            print("Disabling W-projection. Enabling AIPS-style faceting", file=log)
+
         self.DoDDE = False
         if Sols is not None:
             self.setSols(Sols)
@@ -126,7 +129,7 @@ class ClassFacetMachine():
         self.Oversize = Oversize
 
         DecorrMode=self.GD["RIME"]["DecorrMode"]
-        if DecorrMode is not None and DecorrMode is not "":
+        if DecorrMode is not None and DecorrMode != "":
             print(ModColor.Str("Using decorrelation mode %s"%DecorrMode), file=log)
         self.AverageBeamMachine=None
         self.SmoothJonesNorm=None
@@ -259,6 +262,8 @@ class ClassFacetMachine():
         self.Cell = Cell
         self.CellSizeRad = (Cell / 3600.) * np.pi / 180.
         rac, decc = self.VS.ListMS[0].radec
+        
+
         self.MainRaDec = (rac, decc)
         self.nch = self.VS.NFreqBands
         # LB - this is unnecessary and only used in FacetMachine, replacing occurrences
@@ -266,6 +271,19 @@ class ClassFacetMachine():
         self.SumWeights = np.zeros((self.nch, self.npol), float)
 
         self.CoordMachine = ModCoord.ClassCoordConv(rac, decc)
+
+        # from DDFacet.ToolsDir.rad2hmsdms import rad2hmsdms
+        # print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
+        # print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
+        # print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
+        # print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
+        # print(rad2hmsdms(rac,Type="ra").replace(" ",":"),rad2hmsdms(decc,Type="dec").replace(" ","."))
+        # print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
+        # print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
+        # print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
+        # print("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
+
+
         # get the closest fast fft size:
         Npix = self.GD["Image"]["NPix"]
         Padding = self.GD["Facets"]["Padding"]
@@ -282,6 +300,24 @@ class ClassFacetMachine():
                                          [-RadiusTot, RadiusTot]])
         self.setFacetsLocs()
 
+    def giveEffectivePadding(self,iFacet):
+        ThisFacetPadding=self.Padding
+        if self.GD["Facets"].get("FluxPaddingAppModel",None) is None:
+            return ThisFacetPadding
+
+        FFacet=np.array([self.DicoImager[iFacet]["MaxFlux"]+self.DicoImager[iFacet]["TotalFlux"] for iFacet in self.DicoImager.keys()])
+        MaxFluxOverFacets=np.max(FFacet)
+        MedFluxOverFacets=np.median(FFacet)
+        
+        ThisFacetPadding=self.GD["Facets"]["FluxPaddingScale"]*self.Padding * (FFacet-MedFluxOverFacets)/MaxFluxOverFacets
+        
+        ThisFacetPadding=np.max([self.Padding,ThisFacetPadding[iFacet]])
+
+        if ThisFacetPadding!=self.Padding:
+            log.print("Facet #%i padding %.2f -> %.2f"%(iFacet,self.Padding,ThisFacetPadding))
+        
+        return ThisFacetPadding
+        
     def AppendFacet(self, iFacet, l0, m0, diam):
         """
         Adds facet dimentions to info dict of facets (self.DicoImager[iFacet])
@@ -302,11 +338,14 @@ class ClassFacetMachine():
         # print>>log,"Facet %d l %f m %f RA %f Dec %f"%(iFacet, l0, m0, raFacet, decFacet)
 
         NpixFacet, _ = EstimateNpix(diam / self.CellSizeRad, Padding=1)
-        _, NpixPaddedGrid = EstimateNpix(NpixFacet, Padding=self.Padding)
+
+        Padding=self.giveEffectivePadding(iFacet)
+            
+        _, NpixPaddedGrid = EstimateNpix(NpixFacet, Padding=Padding)
 
         if NpixPaddedGrid // NpixFacet > self.Padding and not getattr(self, '_warned_small_ffts', False):
             print(ModColor.Str("WARNING: Your FFTs are too small. We will pad them by x%.2f "\
-                               "rather than x%.2f. Increase facet size and/or padding to get rid of this message." % (float(NpixPaddedGrid)/NpixFacet, self.Padding)),
+                               "rather than x%.2f. Increase facet size and/or padding to get rid of this message." % (float(NpixPaddedGrid)/NpixFacet, Padding)),
                                file=log)
             self._warned_small_ffts = True
 
@@ -332,7 +371,6 @@ class ClassFacetMachine():
         self.DicoImager[iFacet]["lmSol"] = lSol[iSol], mSol[iSol]
         self.DicoImager[iFacet]["radecSol"] = raSol[iSol], decSol[iSol]
         self.DicoImager[iFacet]["iSol"] = iSol
-
         DicoConfigGM = {"NPix": NpixFacet,
                         "Cell": self.GD["Image"]["Cell"],
                         "ChanFreq": self.ChanFreq,
@@ -342,7 +380,10 @@ class ClassFacetMachine():
                         "Nw": self.GD["CF"]["Nw"],
                         "WProj": True,
                         "DoDDE": self.DoDDE,
-                        "Padding": self.GD["Facets"]["Padding"]}
+                        "Padding": Padding}
+
+        
+        #log.print("[%i] %f"%(iFacet,Padding))
 
         _, _, NpixOutIm, NpixOutIm = self.OutImShape
 
@@ -369,6 +410,7 @@ class ClassFacetMachine():
         # self.JonesDirCat.m[iFacet] = m
         # self.JonesDirCat.Cluster[iFacet] = iFacet
 
+        
     def setFacetsLocs(self):
         """
         Routine to split the image into a grid of squares.
@@ -613,7 +655,12 @@ class ClassFacetMachine():
         if self.GD["RIME"]["FFTMachine"]!="FFTW": return
         self.wisdom_cache_path = self.GD["Cache"]["DirWisdomFFTW"]
         #hostname=socket.gethostname()
-        cpuname=cpuinfo.get_cpu_info()["brand"].replace(" ","")
+        #work round incompatible change in cpuinfo
+        cpudict=cpuinfo.get_cpu_info()
+        if 'brand' in cpudict:
+            cpuname=cpudict["brand"].replace(" ","")
+        else:
+            cpuname=cpudict["brand_raw"].replace(" ","")            
         if "~" in self.wisdom_cache_path:
             home = expanduser("~")        
             self.wisdom_cache_path=self.wisdom_cache_path.replace("~",home)
@@ -636,7 +683,7 @@ class ClassFacetMachine():
                 pyfftw.import_wisdom(DictWisdom["Wisdom"])
                 WisdomTypes=DictWisdom["WisdomTypes"]
             except:
-                print("Exception while reading wisdom... will remake")
+                log.print("Exception while reading wisdom... will remake")
                 WisdomTypes=[]
         else:
             WisdomTypes=[]
@@ -686,6 +733,7 @@ class ClassFacetMachine():
         else:
             wmax = self.VS.getMaxW()
             print("max w=%.6g from MS (--CF-wmax=0)"%wmax, file=log)
+
         # subprocesses will place W-terms etc. here. Reset this first.
         self._CF = shared_dict.create("CFPSF" if self.DoPSF else "CF")
         # check if w-kernels, spacial weights, etc. are cached
@@ -711,7 +759,7 @@ class ClassFacetMachine():
         for iFacet in getattr(self.DicoImager, "iterkeys", self.DicoImager.keys)():
             facet_dict = self._CF.addSubdict(iFacet)
             APP.runJob("%s.InitCF.f%s"%(self._app_id, iFacet), self._initcf_worker,
-                            args=(iFacet, facet_dict.readwrite(), cachepath, cachevalid, wmax))
+                            args=(iFacet, facet_dict.readwrite(), cachepath, cachevalid, wmax))#,serial=True)
         #workers_res=APP.awaitJobResults("%s.InitCF.*"%self._app_id, progress="Init CFs")
 
 
@@ -827,6 +875,7 @@ class ClassFacetMachine():
             iFacet, self.SpheNorm, self.VS.NFreqBands,
             self.StokesConverter.AvailableCorrelationProductsIds(),
             self.StokesConverter.RequiredStokesProductsIds(),
+            Padding=FacetInfo["DicoConfigGM"]["Padding"],
             **kw)
 
     def ToCasaImage(self, ImageIn, Fits=True, ImageName=None,
@@ -1121,15 +1170,23 @@ class ClassFacetMachine():
 
             ## OMS: see issue #484. Restructuring this to use shm, and less of it. Note that the only user
             ## of this structure is ClassSpectralFunctions and ClassPSFServer
-            ## [iMS][iFacet,0,:] is the sum of the per-channel weights
+            ## [iMS][iFacet,0,:] is the sum of the per-channel Jones
             ## [iMS][iFacet,1,:] is the sum of the per-channel weights squared
             ListSumJonesChan = DicoImages.addSubdict("SumJonesChan")
+
+            HasNoBeam = ((self.GD["Beam"]["Model"] is None) or (self.GD["Beam"]["Model"]==""))
+            HasNoDDESolutions = ((self.GD["DDESolutions"]["DDSols"] is None) or (self.GD["DDESolutions"]["DDSols"]==""))
+            
             for iMS in range(self.VS.nMS):
                 nVisChan = self.VS.ListMS[iMS].ChanFreq.size
                 ThisMSSumJonesChan = ListSumJonesChan.addSharedArray(iMS, (len(facets), 2, nVisChan), np.float64)
                 for iFacet in facets:
                     sumjones = self.DicoImager[iFacet]["SumJonesChan"][iMS]
-                    sumjones[sumjones == 0] = 1.
+
+                    if HasNoBeam and HasNoDDESolutions:
+                        sumjones[sumjones == 0] = 1.
+                    if np.all(sumjones==0):
+                        log.print(ModColor.Str("MS #%i facet #%i has zero SumJonesChan for all channels"%(iMS,iFacet)))
                     ThisMSSumJonesChan[iFacet,:] = sumjones[:]
 
             DicoImages["ChanMappingGrid"] = self.VS.DicoMSChanMapping
@@ -1318,6 +1375,7 @@ class ClassFacetMachine():
                     # ThisSumWeights.reshape((nch,npol,1,1))[Channel, pol, 0, 0]
                     if kind == "Jones-amplitude":
                         Im = SpacialWeigth[::-1, :].T[x0facet:x1facet, y0facet:y1facet] * ThisSumJones
+                        # print(iFacet,Channel,ThisSumJones)
                     else:
                         if kind == "Dirty" or kind == "PSF":
                             # make copy since subsequent operations are in-place
@@ -1477,7 +1535,9 @@ class ClassFacetMachine():
         # Create Jones Matrices Dictionary
         DicoJonesMatrices = None
         Apply_killMS = self.GD["DDESolutions"]["DDSols"]
-        Apply_Beam = self.GD["Beam"]["Model"] is not None
+        Apply_Beam = self.GD["Beam"]["Model"] is not None and \
+                     self.GD["Beam"]["Model"] != "" and \
+                     self.GD["Beam"]["Model"] != 0
 
         if Apply_killMS or Apply_Beam:
             DicoJonesMatrices = {}
@@ -1547,7 +1607,7 @@ class ClassFacetMachine():
         for iDir in range(self.AverageBeamMachine.NDir):
             APP.runJob("%s%d" % (JobName,iDir), 
                        self._SmoothAverageBeam_worker,
-                       args=(DATA.readonly(), iDir))
+                       args=(DATA.readonly(), iDir))#,serial=True)
 
 
     def finaliseSmoothBeam(self):
@@ -1668,7 +1728,7 @@ class ClassFacetMachine():
         APP.awaitJobResults(self._fft_job_id+"*", progress=("FFT PSF" if self.DoPSF else "FFT"))
         self._fft_job_id = None
 
-    def _set_model_grid_worker(self, iFacet, model_dict, cf_dict, ChanSel, ToSHMDict=False,ToGrid=False,ApplyNorm=True):
+    def _set_model_grid_worker(self, iFacet, model_dict, cf_dict, ChanSel, ToSHMDict=False,ToGrid=False,ApplyNorm=True,DoReturn=True):
         # We get the psf dict directly from the shared dict name (not from the .path of a SharedDict)
         # because this facet machine is not necessarilly the one where we have computed the PSF
         norm_dict = shared_dict.attach("normDict")
@@ -1680,7 +1740,8 @@ class ClassFacetMachine():
         model_dict[iFacet]["SumFlux"] = SumFlux
         if ToSHMDict:
             model_dict[iFacet]["FacetGrid"] = ModelGrid
-        return ModelGrid
+        if DoReturn:
+            return ModelGrid
 
     def set_model_grid (self,ToGrid=True,ApplyNorm=True):
         self.awaitInitCompletion()
@@ -1701,9 +1762,9 @@ class ClassFacetMachine():
             APP.runJob("%sF%d" % (self._set_model_grid_job_id, iFacet), 
                        self._set_model_grid_worker,
                        args=(iFacet, self._model_dict.readwrite(), self._CF[iFacet].readonly(),
-                             ChanSel,ToSHMDict,ToGrid,ApplyNorm))
+                             ChanSel,ToSHMDict,ToGrid,ApplyNorm,False))
         APP.awaitJobResults(self._set_model_grid_job_id + "*", progress="Make model grids")
-
+        del(self._model_dict["Image"])
 
     # #####################################################"
     def _convolveShift_worker(self, iFacet, d_mat, dl,dm,
