@@ -30,8 +30,16 @@ from DDFacet.Other import MyPickle
 from scipy.spatial import Voronoi, ConvexHull
 from SkyModel.Sky import ModVoronoi
 from DDFacet.Other import reformat
+from DDFacet.Other import ModColor
+from DDFacet.Imager.ModModelMachine import ClassModModelMachine
+from DDFacet.Other import Exceptions
+
+from DDFacet.Data.ClassJones import _parse_solsfile, _which_solsfile
+import os, glob
+
 from DDFacet.Data.ClassJones import _parse_solsfile
 import os
+
 from DDFacet.ToolsDir.ModToolBox import EstimateNpix
 import tables
 
@@ -41,6 +49,7 @@ import Polygon
 from DDFacet.ToolsDir import rad2hmsdms
 from DDFacet.Other import logger
 log = logger.getLogger("ClassFacetMachineTessel")
+from pyrap.images import image
 
 
 class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
@@ -84,13 +93,15 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
         #         MSName.append(ll)
         #     MSName = MSName[0]
 
-
         MSName = self.VS.ListMS[0].MSName
 
         SolsFile = self.GD["DDESolutions"]["DDSols"]
         if isinstance(SolsFile, list):
             SolsFile = self.GD["DDESolutions"]["DDSols"][0]
 
+        if SolsFile=="": SolsFile=None
+
+
         if SolsFile and (not (".npz" in SolsFile)) and (not (".h5" in SolsFile)):
             Method = SolsFile
             # ThisMSName = reformat.reformat(
@@ -107,8 +118,81 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
                     os.makedirs(DirName)
                 SolsFile = "%s/killMS.%s.sols.npz"%(DirName,SolsFile)
 
+        
+        
+        
+        if SolsFile is not None and not ".h5" in SolsFile:
+            DoCheckParset=False
+            if "killMS" in SolsFile:
+                try:
+                    from killMS.Parset import ReadCFG as ReadCFGkMS
+                    PName=SolsFile[0:-4]+".parset"
+                    DoCheckParset=os.path.isfile(PName)
+                    if not DoCheckParset:
+                        log.print(ModColor.Str("File %s does not exist - can't check parset consistencies"%PName))
+                except:
+                    log.print(ModColor.Str("Cannot import killMS - can't check parset consistencies"))
+                    DoCheckParset=False
+                    
+            if DoCheckParset:
+                print("reading %s"%PName,file=log)
+                GDkMS=ReadCFGkMS.Parset(PName).DicoPars
+                DoCheckParset=False
+                
+            
+                Ls=[]
+            
+                def CheckField(D0,k0,D1,k1):
+                    try:
+                        v0=GDkMS[D0][k0]
+                        v1=self.GD[D1][k1]
+                        if v0=="": v0=None
+                        if v1=="": v1=None
+                        if v0!=v1:
+                            Ls.append("!!! kMS parameter [[%s][%s] = %s] differs from DDF [[%s][%s] = %s]"%(D0,k0,str(v0),D1,k1,str(v1)))
+                    except:
+                        pass
+                
+            
+                CheckField("Beam",'BeamModel',"Beam","Model")
+                CheckField("Beam",'NChanBeamPerMS',"Beam","NBand")
+                CheckField("Beam",'BeamAt', "Beam","At") # tessel/facet
+                CheckField("Beam",'LOFARBeamMode', "Beam","LOFARBeamMode")     # A/AE
+                CheckField("Beam",'DtBeamMin', "Beam","DtBeamMin")
+                CheckField("Beam",'CenterNorm', "Beam","CenterNorm")
+                CheckField("Beam",'FITSFile', "Beam","FITSFile")
+                CheckField("Beam",'FITSParAngleIncDeg', "Beam","FITSParAngleIncDeg")
+                CheckField("Beam",'FITSLAxis', "Beam","FITSLAxis")
+                CheckField("Beam",'FITSMAxis', "Beam","FITSMAxis")
+                CheckField("Beam",'FITSFeed', "Beam","FITSFeed") 
+                # CheckField("Beam",'FITSVerbosity', "Beam","FITSVerbosity")
+                CheckField("Beam","FeedAngle", "Beam","FeedAngle")
+                CheckField("Beam",'ApplyPJones', "Beam","ApplyPJones")
+                CheckField("Beam",'FlipVisibilityHands', "Beam","FlipVisibilityHands")
+                CheckField("Beam",'FITSFeedSwap',"Beam","FITSFeedSwap")
+    
+                CheckField("ImageSkyModel",'MaxFacetSize',"Facets","DiamMax")
+                CheckField("ImageSkyModel",'MinFacetSize',"Facets","DiamMin")
+                CheckField("SkyModel",'Decorrelation',"RIME","DecorrMode")
+    
+                if len(Ls)>0:
+                    log.print(ModColor.Str("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"))
+                    log.print(ModColor.Str("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"))
+                    log.print(ModColor.Str("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"))
+                    log.print(ModColor.Str("!!! The following parameters are different in kMS/DDF, and you may think whether this has an effect or not..."))
+                    for l in Ls:
+                        log.print(ModColor.Str(l))
+                    log.print(ModColor.Str("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"))
+                    log.print(ModColor.Str("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"))
+                    log.print(ModColor.Str("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"))
+                else:
+                    log.print(ModColor.Str("All kMS/DDF beam parameters are the same...",col="green"))
+    
+        if (self.GD["Facets"]["CatNodes"] is not None) and (SolsFile is not None):
+            log.print(ModColor.Str("Both --Facets-CatNodes and --DDESolutions-DDSols are specified which might have different clusterings..."))
 
-
+            
+                
 #        if "CatNodes" in self.GD.keys():
         regular_grid = False
         if self.GD["Facets"]["CatNodes"] is not None:
@@ -126,9 +210,13 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
             decNode = ClusterNodes.dec
             lFacet, mFacet = self.CoordMachine.radec2lm(raNode, decNode)
         elif SolsFile is not None and ".h5" in  SolsFile:
-            h5file, apply_solsets, apply_map = _parse_solsfile(SolsFile)
-            log.print("Taking facet directions from H5parm: {}, solsets: {}".format(h5file, apply_solsets))
-            with tables.open_open(h5file) as H:
+
+            h5files, apply_solsets, apply_map = _parse_solsfile(SolsFile)
+            print("Parsing h5file pattern {}".format(h5files), file=log)
+            import glob
+            h5file = glob.glob(h5files)[0]
+            print( "Taking facet directions from H5parm: {}, solsets: {}".format(h5file, apply_solsets), file=log)
+            with tables.open_file(h5file) as H:
                 lm, radec = [], []
                 for solset in apply_solsets:
                     _solset = getattr(H.root, solset)
@@ -219,14 +307,14 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
                 # pylab.draw()
                 # pylab.show()
                 # #pylab.pause(0.1)
-                
+
                 if ThisP.size>0:
                     LPolygon.append(ThisP[0])
                     ListNode.append(iNode)
             NodesCat=NodesCat[np.array(ListNode)].copy()
 
 
-            
+
 # =======
 #             LPolygon = [
 #                 np.array(PP & Polygon.Polygon(np.array(vertices[region])))[0]
@@ -239,9 +327,10 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
         # VM.ToReg(regFile,lFacet,mFacet,radius=.1)
 
         NodeFile = "%s.NodesCat.npy" % self.GD["Output"]["Name"]
-        print("Saving Nodes catalog in %s" % NodeFile, file=log)
+        print("Saving Nodes catalog in %s (Nfacets:%i)" % (NodeFile, NFacets), file=log)
         np.save(NodeFile, NodesCat)
 
+        
         for iFacet, polygon0 in zip(range(len(LPolygon)), LPolygon):
             # polygon0 = vertices[region]
             P = polygon0.tolist()
@@ -266,7 +355,7 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
         DiamMax = self.GD["Facets"]["DiamMax"] * np.pi / 180
         # DiamMax=4.5*np.pi/180
         DiamMin = self.GD["Facets"]["DiamMin"] * np.pi / 180
-
+        
         def ClosePolygon(polygon):
             P = polygon.tolist()
             polygon = np.array(P + [P[0]])
@@ -281,7 +370,7 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
             # polygonFacet=ClosePolygon(polygonFacet)
             P0 = Polygon.Polygon(polygonFacet)
             P0Cut = Polygon.Polygon(P0 & PFOV)
-
+            
             if P0Cut.nPoints() == 0:
                 return []
 
@@ -305,42 +394,80 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
             Lc = Lc.ravel().tolist()
             Mc = Mc.ravel().tolist()
 
-            DpolySquare = np.array(
-                [[-dl, -dm],
-                 [dl, -dm],
-                 [dl, dm],
-                 [-dl, dm]]) * 0.5
+            DpolySquare = np.array([[-dl, -dm],
+                                    [dl, -dm],
+                                    [dl, dm],
+                                    [-dl, dm]]) * 0.5
+            
             for lc, mc in zip(Lc, Mc):
                 polySquare = DpolySquare.copy()  # ClosePolygon(DpolySquare.copy())
                 polySquare[:, 0] += lc
                 polySquare[:, 1] += mc
                 # polySquare=ClosePolygon(polySquare)
                 P1 = Polygon.Polygon(polySquare)
-
+                
                 POut = (P0Cut & P1)
                 if POut.nPoints() == 0:
                     continue
 
-                polyOut = np.array(POut[0])
+                DoPlot=0
+                if len(POut)>1:
+                    DoPlot=1
+                    log.print(ModColor.Str("WARNING: There are more than one polygon in the intersection"))
+                    sFile="Poly%i.npz"%self.iSave
+                    log.print(ModColor.Str("   Saving polygon file for eventual debugging as: %s"%sFile))
+                    np.savez(sFile,
+                             polySquare=polySquare,
+                             polygonFacetCut=polygonFacetCut)
+                    log.print(ModColor.Str("   Taking the largest polygon..."))
+                    A=0
+                    for iP in range(len(POut)):
+                        Aa=Polygon.Polygon(POut[iP]).area()
+                        if Aa>A:
+                            A=Aa
+                            polyOut = np.array(POut[iP])
+                else:
+                    polyOut = np.array(POut[0])
+                    
                 # polyOut=ClosePolygon(polyOut)
                 LPoly.append(polyOut)
 
-                # pylab.clf()
-                # x,y=polygonFacetCut.T
-                # pylab.plot(x,y,color="blue")
-                # x,y=polygonFacet.T
-                # pylab.plot(x,y,color="blue",ls=":",lw=3)
-                # x,y=np.array(PFOV[0]).T
-                # pylab.plot(x,y,color="black")
-                # x,y=polySquare.T
-                # pylab.plot(x,y,color="green",ls=":",lw=3)
-                # x,y=polyOut.T
-                # pylab.plot(x,y,color="red",ls="--",lw=3)
-                # pylab.xlim(-0.03,0.03)
-                # pylab.ylim(-0.03,0.03)
-                # pylab.draw()
-                # pylab.show(False)
-                # pylab.pause(0.5)
+                # ################################
+                # def CloseLoop(x,y):
+                #     x=np.concatenate([x,np.array([x[0]])])
+                #     y=np.concatenate([y,np.array([y[0]])])
+                #     return x,y
+                # if False:#DoPlot:
+                #     # np.savez("Poly%i.npz"%self.iSave,
+                #     #          polySquare=polySquare,
+                #     #          polygonFacetCut=polygonFacetCut)
+                #     import pylab
+                #     # pylab.clf()
+                #     x,y=polygonFacetCut.T
+                #     x,y=CloseLoop(x,y)
+                #     #pylab.plot(x,y,color="blue")
+                #     x,y=polygonFacet.T
+                #     x,y=CloseLoop(x,y)
+                #     #pylab.plot(x,y,color="blue",ls=":",lw=3)
+                #     x,y=np.array(PFOV[0]).T
+                #     x,y=CloseLoop(x,y)
+                #     pylab.plot(x,y,color="black")
+                #     x,y=polySquare.T
+                #     x,y=CloseLoop(x,y)
+                #     pylab.plot(x,y,color="green",ls=":",lw=3)
+                #     x,y=polyOut.T
+                #     x,y=CloseLoop(x,y)
+                #     pylab.plot(x,y,color="red",ls="--",lw=3)
+                #     pylab.scatter(x,y,c="red")
+                #     pylab.xlim(self.RadiusTot,-self.RadiusTot)
+                #     pylab.ylim(-self.RadiusTot,self.RadiusTot)
+                #     pylab.title("iSave=%i"%self.iSave)
+                #     pylab.draw()
+                #     pylab.show(block=True)
+                #     #pylab.pause(0.5)
+                # #############################
+                
+                self.iSave+=1
 
             return LPoly
 
@@ -351,6 +478,7 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
 
         LPolygonNew = []
 
+        self.iSave=0
         for iFacet in range(len(LPolygon)):
             polygon = LPolygon[iFacet]
             ThisDiamMax = DiamMax
@@ -362,6 +490,8 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
         # VM.PolygonToReg(regFile,LPolygonNew,radius=0.1,Col="green",labels=[str(i) for i in range(len(LPolygonNew))])
 
         DicoPolygon = {}
+        # import pylab
+        # Lx,Ly=[],[]
         for iFacet in range(len(LPolygonNew)):
             DicoPolygon[iFacet] = {}
             poly = LPolygonNew[iFacet]
@@ -372,11 +502,31 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
             xc, yc = np.mean(poly[:, 0]), np.mean(poly[:, 1])
             DicoPolygon[iFacet]["xyc"] = xc, yc
             dSol = np.sqrt((xc - lFacet) ** 2 + (yc - mFacet) ** 2)
-            DicoPolygon[iFacet]["iSol"] = np.where(dSol == np.min(dSol))[0]
+            ind=np.where(dSol == np.min(dSol))[0][0:1]
+            DicoPolygon[iFacet]["iSol"] = ind
+            # if ind.size==2: stop
+            # Lx.append(xc)
+            # Ly.append(yc)
+            # print(iFacet,DicoPolygon[iFacet]["iSol"])
+            
 
+        # for iFacet in list(DicoPolygon.keys()):
+        #     pylab.clf()
+        #     pylab.scatter(lFacet,mFacet,c="red")
+        #     pylab.scatter(Lx,Ly,c="blue")
+        #     ind=DicoPolygon[iFacet]["iSol"]
+        #     pylab.scatter(lFacet[ind],mFacet[ind],c="green")
+        #     pylab.scatter(Lx[iFacet],Ly[iFacet],c="black")
+        #     pylab.draw()
+        #     pylab.show(block=False)
+        #     pylab.pause(0.1)
+        #     if ind.size==2: stop
+
+
+            
         for iFacet in sorted(DicoPolygon.keys()):
             diam = DicoPolygon[iFacet]["diamMin"]
-            # print iFacet,diam,DiamMin
+            #print(iFacet,diam,DiamMin)
             if diam < DiamMin:
                 dmin = 1e6
                 xc0, yc0 = DicoPolygon[iFacet]["xyc"]
@@ -538,8 +688,77 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
 
         for iFacet in range(l_m_Diam.shape[0]):
             self.DicoImager[iFacet] = {}
-            self.DicoImager[iFacet]["Polygon"] = D[
-                l_m_Diam[iFacet, 3]]["Polygon"]
+            self.DicoImager[iFacet]["Polygon"] = D[l_m_Diam[iFacet, 3]]["Polygon"]
+            
+
+        if self.GD["Facets"].get("FluxPaddingAppModel",None) is not None:
+            NameModel=self.GD["Facets"]["FluxPaddingAppModel"]
+            log.print("Computing individual facet flux density for facet-dependent padding...")
+            ModelImage=image(NameModel).getdata()
+            nch,npol,_,_=ModelImage.shape
+            ModelImage=np.mean(ModelImage[:,0,:,:],axis=0)
+            ModelImage=(ModelImage.T[::-1]).copy()
+            _,_,nx,ny=self.OutImShape
+            Dx=self.CellSizeRad * nx/2
+            Dy=self.CellSizeRad * ny/2
+
+            from DDFacet.Other import ClassTimeIt
+            T=ClassTimeIt.ClassTimeIt("FluxPaddingAppModel")
+            T.disable()
+            lg, mg = X, Y = np.mgrid[-Dx:Dx:nx * 1j, -Dy:Dy:ny * 1j]
+            for iFacet in self.DicoImager.keys():
+                vertices = self.DicoImager[iFacet]["Polygon"]
+                lp,mp=vertices.T
+
+                # indx=((lg>=lp.min())&(lg<lp.max()))
+                # indy=((mg>=mp.min())&(mg<mp.max()))
+                # ind=(indx&indy)
+                
+                lg1, mg1 = np.mgrid[-Dx:Dx:nx * 1j], np.mgrid[-Dy:Dy:ny * 1j]
+                indx=((lg1>=lp.min())&(lg1<lp.max()))
+                indy=((mg1>=mp.min())&(mg1<mp.max()))
+                ind1=(indx.reshape((-1,1))*indy.reshape((1,-1)))
+                # print(np.allclose(ind,ind1))
+                # stop
+                ind=ind1
+                
+                ModelImage_s=ModelImage[ind]
+                X,Y=lgs,mgs=lg[ind],mg[ind]
+                
+                XY = np.dstack((X, Y))
+                XY_flat = XY.reshape((-1, 2))
+                T.timeit("build s")
+                
+                mpath = Path(vertices)  # the vertices of the polygon
+                mask_flat = mpath.contains_points(XY_flat)
+                T.timeit("contains")
+                mask = mask_flat.reshape(X.shape)
+                Ft=np.max(ModelImage_s.flat[mask.ravel()])
+                # log.print("Flux Facet [%.3i] = %f"%(iFacet,Ft))
+                self.DicoImager[iFacet]["MaxFlux"]=Ft
+                Ft=np.sum(ModelImage_s.flat[mask.ravel()])
+                self.DicoImager[iFacet]["TotalFlux"]=Ft
+                T.timeit("rest")
+
+            # lg, mg = X, Y = np.mgrid[-Dx:Dx:nx * 1j, -Dy:Dy:ny * 1j]
+            # XY = np.dstack((X, Y))
+            # XY_flat = XY.reshape((-1, 2))
+            # for iFacet in self.DicoImager.keys():
+            #     vertices = self.DicoImager[iFacet]["Polygon"]
+            #     mpath = Path(vertices)  # the vertices of the polygon
+            #     mask_flat = mpath.contains_points(XY_flat)
+            #     T.timeit("contains")
+            #     mask = mask_flat.reshape(X.shape)
+            #     Ft=np.max(ModelImage.flat[mask.ravel()])
+            #     log.print("Flux Facet [%.3i] = %f"%(iFacet,Ft))
+            #     self.DicoImager[iFacet]["MaxFlux"]=Ft
+            #     Ft=np.sum(ModelImage.flat[mask.ravel()])
+            #     self.DicoImager[iFacet]["TotalFlux"]=Ft
+            #     T.timeit("rest")
+
+        ###############
+                
+        for iFacet in range(l_m_Diam.shape[0]):
             x0 = round(l_m_Diam[iFacet, 0] / self.CellSizeRad)
             y0 = round(l_m_Diam[iFacet, 1] / self.CellSizeRad)
             # if x0 % 2 == 0:
@@ -555,6 +774,10 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
             # self.AppendFacet(iFacet,l0,m0,diam)
             self.AppendFacet(iFacet, l0, m0, diam)
 
+
+
+            
+
         # self.MakeMasksTessel()
 
         NpixMax = np.max([self.DicoImager[iFacet]["NpixFacet"]
@@ -615,8 +838,9 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
             indx,indy=np.where(R==np.min(R))
             lmin,mmin=X[indx[0],indy[0]],Y[indx[0],indy[0]]
             self.DicoImager[iFacet]["lm_min"]=lmin,mmin
-            
 
+
+            
         self.FacetDirCat = np.zeros((len(self.DicoImager),),
                                     dtype=[('Name', '|S200'),
                                            ('ra', np.float),
@@ -653,53 +877,3 @@ class ClassFacetMachineTessel(ClassFacetMachine.ClassFacetMachine):
             ss = "%s, %s, %f, %f" % (sra, sdec,ra,dec)
             f.write(ss+'\n')
         f.close()
-
-# <<<<<<< HEAD
-# #===============================================
-# #===============================================
-# #===============================================
-# #===============================================
-
-
-# class WorkerImager(ClassFacetMachine.WorkerImager):
-#     def init(self, DicoJob):
-#         iFacet=DicoJob["iFacet"]
-#         #Create smoothned facet tessel mask:
-#         Npix = self.DicoImager[iFacet]["NpixFacetPadded"]
-#         l0, l1, m0, m1 = self.DicoImager[iFacet]["lmExtentPadded"]
-#         X, Y = np.mgrid[l0:l1:Npix * 1j, m0:m1:Npix * 1j]
-#         XY = np.dstack((X, Y))
-#         XY_flat = XY.reshape((-1, 2))
-#         vertices = self.DicoImager[iFacet]["Polygon"]
-#         mpath = Path(vertices)  # the vertices of the polygon
-#         mask_flat = mpath.contains_points(XY_flat)
-
-
-
-#         mask = mask_flat.reshape(X.shape)
-
-#         mpath = Path(self.CornersImageTot)
-#         mask_flat2 = mpath.contains_points(XY_flat)
-#         mask2 = mask_flat2.reshape(X.shape)
-#         mask[mask2 == 0] = 0
-
-
-#         GaussPars = (10, 10, 0)
-
-#         SpacialWeigth = np.float32(mask.reshape((1, 1, Npix, Npix)))
-#         SpacialWeigth = ModFFTW.ConvolveGaussian(SpacialWeigth, CellSizeRad=1, GaussPars=[GaussPars])
-#         SpacialWeigth = SpacialWeigth.reshape((Npix, Npix))
-#         SpacialWeigth /= np.max(SpacialWeigth)
-#         NameSpacialWeigth = "%sSpacialWeight.Facet_%3.3i" % (self.FacetDataCache, iFacet)
-#         NpShared.ToShared(NameSpacialWeigth, SpacialWeigth)
-#         #Initialize a grid machine per facet:
-#         self.GiveGM(iFacet)
-#         self.result_queue.put({"Success": True, "iFacet": iFacet})
-
-
-
-
-
-# =======
-# >>>>>>> issue-255
-
