@@ -163,16 +163,16 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
         at the same position, but different fluxes, alphas etc.
 
         """
-            
+
         DicoComp = self.DicoSMStacked["Comp"]
         ref_freq = self.DicoSMStacked["RefFreq"]
-        
+
         if FreqIn is None:
            FreqIn=np.array([ref_freq], dtype=np.float32)
 
         # Assumptions:
         # DicoSMStacked is a dictionary of "Solution" dictionaries
-        # keyed on (l, m), corresponding to some point  source. 
+        # keyed on (l, m), corresponding to some point  source.
         # Components associated with the source for each scale are
         # located in self.ListScales.
 
@@ -258,6 +258,19 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
         Fs = np.fft.fftshift
         iFs = np.fft.ifftshift
 
+        import pyfftw
+        nthreads = int(self.GD['Parallel']['NCPU'])
+        if not nthreads:
+            import multiprocessing
+            nthreads = multiprocessing.cpu_count()
+        else:
+            from multiprocessing.pool import ThreadPool
+            import dask
+
+            dask.config.set(pool=ThreadPool(nthreads))
+        FFT = lambda x: pyfftw.interfaces.numpy_fft.fft2(x, axes=(-2, -1), planner_effort='FFTW_ESTIMATE', threads=nthreads) #, norm='ortho')
+        iFFT = lambda x: pyfftw.interfaces.numpy_fft.ifft2(x, axes=(-2, -1), planner_effort='FFTW_ESTIMATE', threads=nthreads) #, norm='ortho')
+
         # evaluate model
         ModelImage = self.GiveModelImage(self.GridFreqs)
 
@@ -265,7 +278,8 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
         GaussKern = np.pad(GaussKern, self.Npad, mode='constant')
         FTshape, _ = GaussKern.shape
         from scipy import fftpack as FT
-        GaussKernhat = FT.fft2(iFs(GaussKern))
+        #GaussKernhat = FT.fft2(iFs(GaussKern))
+        GaussKernhat = FFT(iFs(GaussKern))
 
         # pad and FT of ModelImage
         ModelImagehat = np.zeros((self.Nchan, FTshape, FTshape), dtype=np.complex128)
@@ -273,11 +287,13 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
         I = slice(self.Npad, -self.Npad)
         for i in range(self.Nchan):
             tmp_array = np.pad(ModelImage[i, 0], self.Npad, mode='constant')
-            ModelImagehat[i] = FT.fft2(iFs(tmp_array)) * GaussKernhat
-            ConvModelImage[i] = Fs(FT.ifft2(ModelImagehat[i]))[I, I].real
+            # ModelImagehat[i] = FT.fft2(iFs(tmp_array)) * GaussKernhat
+            ModelImagehat[i] = FFT(iFs(tmp_array)) * GaussKernhat
+            # ConvModelImage[i] = Fs(FT.ifft2(ModelImagehat[i]))[I, I].real
+            ConvModelImage[i] = Fs(iFFT(ModelImagehat[i]))[I, I].real
 
         if ResidCube is not None:
-            ConvModelImage += ResidCube.squeeze()
+            #ConvModelImage += ResidCube.squeeze()
             RMS = np.std(ResidCube.flatten())
             Threshold = self.GD["SPIMaps"]["AlphaThreshold"] * RMS
         else:
@@ -304,31 +320,17 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
         try:
             import traceback
             from africanus.model.spi.dask import fit_spi_components
-            NCPU = self.GD["Parallel"]["NCPU"]
-            if NCPU:
-                from multiprocessing.pool import ThreadPool
-                import dask
-
-                dask.config.set(pool=ThreadPool(NCPU))
-            else:
-                import multiprocessing
-                NCPU = multiprocessing.cpu_count()
-
             import dask.array as da
             _, ncomps = FitCube.shape
-            FitCubeDask = da.from_array(FitCube.T.astype(np.float64), chunks=(ncomps//NCPU, self.Nchan))
+            FitCubeDask = da.from_array(FitCube.T.astype(np.float64),
+                                        chunks=(np.maximum(100, ncomps//nthreads), self.Nchan))
             weightsDask = da.from_array(weights.astype(np.float64), chunks=(self.Nchan))
             freqsDask = da.from_array(np.array(self.GridFreqs).astype(np.float64), chunks=(self.Nchan))
 
             alpha, varalpha, Iref, varIref = fit_spi_components(FitCubeDask, weightsDask,
-                                                                freqsDask, self.RefFreq,
-                                                                dtype=np.float64).compute()
+                                                                freqsDask, self.RefFreq).compute()
         except Exception as e:
-            traceback_str = traceback.format_exc(e)
-            print("Warning - Failed at importing africanus spi fitter. This could be an issue with the dask " \
-                        "version. Falling back to (slow) scipy version", file=log)
-            print("Original traceback - ", traceback_str, file=log)
-            alpha, varalpha, Iref, varIref = self.FreqMachine.FitSPIComponents(FitCube, self.GridFreqs, self.RefFreq)
+            raise(e)
 
         _, _, nx, ny = ModelImage.shape
         alphamap = np.zeros([nx, ny])
@@ -386,7 +388,7 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
         """ Makes a numpy model for use for killms calibration using SkyModel/MakeModel.py """
         nchan, npol, nx, ny = self.ModelShape
         assert nx == ny, "Only works with square images for now"
-        self.Npix = nx 
+        self.Npix = nx
         self.Nchan = len(self.GridFreqs)
         self.NpixPadded = int(np.ceil(self.GD["Facets"]["Padding"] * self.Npix))
         # make sure it is odd numbered
