@@ -340,17 +340,14 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
                         if ThisComp["ModelType"]=="Delta":
                             ModelImage[ch,pol,x,y]+=Flux
 
-                        elif ThisComp["ModelType"]=="Gaussian":
+                        elif ThisComp["ModelType"]=="Gaussian" or ThisComp["ModelType"]=="Hat":
                             Gauss=ThisComp["Model"]
                             Sup,_=Gauss.shape
                             x0,x1=x-Sup//2,x+Sup//2+1
                             y0,y1=y-Sup//2,y+Sup//2+1
-
-
                             Aedge,Bedge=GiveEdgesDissymetric(x,y,N0x,N0y,Sup//2,Sup//2,Sup,Sup)
                             x0d,x1d,y0d,y1d=Aedge
                             x0p,x1p,y0p,y1p=Bedge
-
                             ModelImage[ch,pol,x0d:x1d,y0d:y1d]+=Gauss[x0p:x1p,y0p:y1p]*Flux
 
         # vmin,vmax=np.min(self._MeanDirtyOrig[0,0]),np.max(self._MeanDirtyOrig[0,0])
@@ -394,11 +391,19 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
             except:
                 print("  Component at (%i, %i) not in dict "%key, file=log)
 
-    def CleanMaskedComponants(self,MaskName):
+    def CleanMaskedComponants(self,MaskName,InvertMask=False):
         print("Cleaning model dictionary from masked components using %s"%(MaskName), file=log)
-        im=image(MaskName)
+        cwd = os.getcwd()
+        baseFitsFile = os.path.basename(MaskName)
+        dirname = os.path.dirname(os.path.abspath(MaskName))
+        os.chdir(dirname)
+        im=image(baseFitsFile)
         MaskArray=im.getdata()[0,0].T[::-1]
-        for (x,y) in self.DicoSMStacked["Comp"].keys():
+        if InvertMask:
+            print("  Inverting the mask", file=log)
+            MaskArray=1-MaskArray
+        os.chdir(cwd)
+        for (x,y) in self.DicoSMStacked["Comp"].copy().keys():
             if MaskArray[x,y]==0:
                 del(self.DicoSMStacked["Comp"][(x,y)])
 
@@ -421,8 +426,13 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
 
         if BeamImage is not None:
             ModelMap*=(BeamImage)
-
-        im=image(FitsFile)
+        
+        cwd = os.getcwd()
+        baseFitsFile = os.path.basename(FitsFile)
+        dirname = os.path.dirname(os.path.abspath(FitsFile))
+        os.chdir(dirname)
+        im=image(baseFitsFile)
+        os.chdir(cwd)
         pol,freq,decc,rac=im.toworld((0,0,0,0))
 
         Lx,Ly=np.where(ModelMap[0,0]!=0)
