@@ -27,6 +27,7 @@ from DDFacet.compatibility import range
 import os
 import numpy as np
 import multiprocessing
+import queue
 import time
 from DDFacet.Other import logger
 from DDFacet.Other import ModColor
@@ -41,6 +42,7 @@ from DDFacet.Imager.SSD.MCMC.ClassMetropolis import ClassMetropolis
 from DDFacet.Array import NpParallel
 from DDFacet.Imager.SSD import ClassIslandDistanceMachine
 from DDFacet.Array import shared_dict
+import psutil
 
 logger.setSilent("ClassArrayMethodSSD")
 logger.setSilent("ClassIsland")
@@ -63,6 +65,8 @@ class ClassImageDeconvMachine():
         self.ModelImage=None
         self.MaxMinorIter=MaxMinorIter
         self.NCPU=NCPU
+        if NCPU==0:
+            self.NCPU=int(GD["Parallel"]["NCPU"] or psutil.cpu_count())
         self.Chi2Thr=10000
         self.GD=GD
         if IdSharedMem is None:
@@ -94,20 +98,20 @@ class ClassImageDeconvMachine():
 
         self.DeconvMode="GAClean"
 
-        if self.GD["GAClean"]["InitType"] == "HMP":
+        InitType=self.GD["SSDClean"]["InitType"]
+        if InitType == "HMP":
             from . import ClassInitSSDModelHMP
             self.InitMachine = ClassInitSSDModelHMP.ClassInitSSDModelParallel(self.GD,
                                                                          NFreqBands,RefFreq,
                                                                          MainCache=self.maincache,
                                                                          IdSharedMem=self.IdSharedMem)
-        elif self.GD["GAClean"]["InitType"] == "MORESANE":
-
+        elif InitType == "MORESANE":
             from . import ClassInitSSDModelMoresane
-            self. InitMachine = ClassInitSSDModelMoresane.ClassInitSSDModelParallel(self.GD,
-                                                                                    NFreqBands, RefFreq,
-                                                                                    NCPU=self.NCPU,
-                                                                                    MainCache=self.maincache,
-                                                                                    IdSharedMem=self.IdSharedMem)
+            self.InitMachine = ClassInitSSDModelMoresane.ClassInitSSDModelParallel(self.GD,
+                                                                                   NFreqBands, RefFreq,
+                                                                                   NCPU=self.NCPU,
+                                                                                   MainCache=self.maincache,
+                                                                                   IdSharedMem=self.IdSharedMem)
         else:
             raise ValueError("InitType should be HMP or MORESANE")
         self._init_machine_initialized = False
@@ -163,7 +167,7 @@ class ClassImageDeconvMachine():
         AllDegridFreqs = []
         for i in kwargs["DegridFreqs"].keys():
             AllDegridFreqs.append(kwargs["DegridFreqs"][i])
-        self.DegridFreqs = np.unique(np.asarray(AllDegridFreqs).flatten())
+        self.DegridFreqs = np.unique(np.concatenate(AllDegridFreqs).flatten())
 
 
     def AdaptArrayShape(self,A,Nout):
@@ -252,7 +256,8 @@ class ClassImageDeconvMachine():
         ListIslands=IslandDistanceMachine.CalcCrossIslandFlux(ListIslands)
         ListIslands=IslandDistanceMachine.ConvexifyIsland(ListIslands)
         ListIslands=IslandDistanceMachine.MergeIslands(ListIslands)
-
+        ListIslands=IslandDistanceMachine.BreakLargeIslands(ListIslands)
+        
         self.LabelIslandsImage=IslandDistanceMachine.CalcLabelImage(ListIslands)
 
         self.ListIslands=ListIslands
@@ -264,7 +269,9 @@ class ClassImageDeconvMachine():
         ind=np.argsort(Sz)[::-1]
 
         ListIslandsOut=[self.ListIslands[i] for i in ind]
-        self.ListIslands=ListIslandsOut
+        self.ListIslands=ListIslandsOut#[100::10][0:1]
+
+        self.NIslands=len(self.ListIslands)
 
 
 
@@ -394,6 +401,10 @@ class ClassImageDeconvMachine():
             print(ModColor.Str("    Initial maximum peak %g Jy below threshold, we're done here" % (ThisFlux),col="green" ), file=log)
             return "FluxThreshold", False, False
 
+        if ThisFlux == 0.0:
+            print(ModColor.Str("   Image is devoid of flux / all your data is flagged! STOPPING!", col="green" ), file=log)
+            return "FluxThreshold", False, False
+            
         self.SearchIslands(StopFlux)
         #return None,None,None
         self.InitIslands()
@@ -563,57 +574,52 @@ class ClassImageDeconvMachine():
                 workerlist[ii].run()
 
         iResult=0
-        #print "!!!!!!!!!!!!!!!!!!!!!!!!",iResult,NJobs
-        while iResult < NJobs:
-            DicoResult=None
-            # for result_queue in List_Result_queue:
-            #     if result_queue.qsize()!=0:
-            #         try:
-            #             DicoResult=result_queue.get_nowait()
-                        
-            #             break
-            #         except:
-                        
-            #             pass
-            #         #DicoResult=result_queue.get()
-            #print "!!!!!!!!!!!!!!!!!!!!!!!!! Qsize",result_queue.qsize()
-            if result_queue.qsize()!=0:
+        success=True
+        try:
+            while iResult < NJobs:
+                DicoResult=None
+                if result_queue.qsize()!=0:
+                    try:
+                        DicoResult=result_queue.get_nowait()
+                    except queue.Empty:
+                        time.sleep(.1)
+                        continue 
+                    except Exception as e:
+                        print("The following unhandled exception occured.", file=log)
+                        import traceback
+                        traceback.print_tb(e.__traceback__, file=log)
+                        success = False
+                        break
+
+                if DicoResult is not None and DicoResult["Success"]:
+                    iResult+=1
+                    NDone=iResult
+                    intPercent=int(100*  NDone / float(NJobs))
+                    pBAR.render(NDone,NJobs)
+
+                    iIsland=DicoResult["iIsland"]
+                    island_dict = deconv_dict[iIsland]
+                    island_dict.reload()
+
+                    self.ModelMachine.AppendIsland(ListIslands[iIsland], island_dict["Model"].copy())
+
+                    if DicoResult["HasError"]:
+                        self.ErrorModelMachine.AppendIsland(ThisPixList, ListIslands[iIsland], island_dict["sModel"].copy())
+
+            deconv_dict.delete()
+        finally:
+            for ii in range(NCPU):
                 try:
-                    DicoResult=result_queue.get_nowait()
-                except:
-                    pass
-                    #DicoResult=result_queue.get()
-
-
-            if DicoResult is None:
-                time.sleep(0.05)
-                continue
-
-            iResult+=1
-            NDone=iResult
-            intPercent=int(100*  NDone / float(NJobs))
-            pBAR.render(NDone,NJobs)
-
-            if DicoResult["Success"]:
-                iIsland=DicoResult["iIsland"]
-                island_dict = deconv_dict[iIsland]
-                island_dict.reload()
-
-                self.ModelMachine.AppendIsland(ListIslands[iIsland], island_dict["Model"].copy())
-
-                if DicoResult["HasError"]:
-                    self.ErrorModelMachine.AppendIsland(ThisPixList, ListIslands[iIsland], island_dict["sModel"].copy())
-
-        deconv_dict.delete()
-
-        for ii in range(NCPU):
-            try:
-                workerlist[ii].shutdown()
-                workerlist[ii].terminate()
-                workerlist[ii].join()
-            except:
-                pass
-        
+                    workerlist[ii].shutdown()
+                    workerlist[ii].terminate()
+                    workerlist[ii].join() 
+                except Exception as e:
+                    print("The following unhandled exception occured.", file=log)
+                    import traceback
+                    traceback.print_tb(e.__traceback__, file=log)
+        if not success:
+            raise RuntimeError("Some parallel jobs have failed. Check your log and report the issue if "
+                               "not a memory issue. Bus errors indicate memory allocation errors")
 
 
 
@@ -748,22 +754,19 @@ class WorkerDeconvIsland(multiprocessing.Process):
 
  
     def run(self):
-
-
+        success = True
         while not self.kill_received and self.CondContinue():
-
-            #gc.enable()
             try:
                 iIsland,FacetID,JonesNorm,PixVariance,shdict_path = self.work_queue.get(True,2)
+            except queue.Empty:
+                continue #sync call above 
             except Exception as e:
-                #print "Exception worker: %s"%str(e)
+                print("The following unhandled exception occured.", file=log)
+                import traceback
+                traceback.print_tb(e.__traceback__, file=log)
+                success = False
                 break
 
-            # iIsland=DicoOrder["iIsland"]
-            # FacetID=DicoOrder["FacetID"]
-            
-            # JonesNorm=DicoOrder["JonesNorm"]
-
             island_dict = shared_dict.attach(shdict_path)
 
             ThisPixList = island_dict["Island"].tolist()
@@ -779,24 +782,6 @@ class WorkerDeconvIsland(multiprocessing.Process):
             if dx>0:
                 IncreaseIslandMachine=ClassIncreaseIsland.ClassIncreaseIsland()
                 ListPixData=IncreaseIslandMachine.IncreaseIsland(ListPixData,dx=dx)
-
-
-            # ################################
-            # DicoSave={"Dirty":self._Dirty,
-            #           "PSF":PSF,
-            #           "FreqsInfo":self.FreqsInfo,
-            #           #"DicoMappingDesc":self.PSFServer.DicoMappingDesc,
-            #           "ListPixData":ListPixData,
-            #           "ListPixParms":ListPixParms,
-            #           "IslandBestIndiv":IslandBestIndiv,
-            #           "GD":self.GD,
-            #           "FacetID":FacetID,
-            #           "iIsland":iIsland,"IdSharedMem":self.IdSharedMem}
-            # print "saving"
-            # MyPickle.Save(DicoSave, "SaveTest")
-            # print "saving ok"
-            # ################################
-
             
             if self.DeconvMode=="GAClean":
                 CEv=ClassEvolveGA(self._Dirty,
@@ -836,40 +821,8 @@ class WorkerDeconvIsland(multiprocessing.Process):
 
                 del(CEv)
                 self.result_queue.put({"Success":True,"iIsland":iIsland,"HasError":True})
-
-
-            # # if island lies inside image
-            # try:
-            #     nch=self.FreqsInfo["MeanJonesBand"][FacetID].size
-            #     #WeightMeanJonesBand=self.FreqsInfo["MeanJonesBand"][FacetID].reshape((nch,1,1,1))
-            #     #WeightMueller=WeightMeanJonesBand.ravel()
-            #     #WeightMuellerSignal=np.sqrt(WeightMueller*self.FreqsInfo["WeightChansImages"].ravel())
-
-            #     CEv=ClassEvolveGA(self._Dirty,
-            #                       PSF,
-            #                       self.FreqsInfo,
-            #                       ListPixParms=ListPixParms,
-            #                       ListPixData=ListPixData,
-            #                       iFacet=FacetID,PixVariance=PixVariance,
-            #                       IslandBestIndiv=IslandBestIndiv,#*np.sqrt(JonesNorm),
-            #                       GD=self.GD)
-            #     #,
-            #      #                 WeightFreqBands=WeightMuellerSignal)
-            #     Model=CEv.main(NGen=NGen,NIndiv=NIndiv,DoPlot=False)
-            
-            #     Model=np.array(Model).copy()#/np.sqrt(JonesNorm)
-            #     #Model*=CEv.ArrayMethodsMachine.Gain
-                
-            #     del(CEv)
-                
-            #     NpShared.ToShared("%s.FitIsland_%5.5i"%(self.IdSharedMem,iIsland),Model)
-                
-            #     #print "Current process: %s [%s left]"%(str(multiprocessing.current_process()),str(self.work_queue.qsize()))
-                
-            #     self.result_queue.put({"Success":True,"iIsland":iIsland})
-            # except Exception,e:
-            #     print "Exception on island %i: %s"%(iIsland,str(e))
-
-            #     self.result_queue.put({"Success":False})
-
-        #print "WORKER DONE"
+            else:
+                raise ValueError("Unknown Deconv mode. This is a bug. Please report")
+        if not success:
+            raise RuntimeError("Some parallel jobs have failed. Check your log and report the issue if "
+                               "not a memory issue. Bus errors indicate memory allocation errors")
\ No newline at end of file
