@@ -18,7 +18,7 @@ from DDFacet.ToolsDir.GiveEdges import GiveEdgesDissymetric
 from scipy.spatial import ConvexHull
 from matplotlib.path import Path
 import psutil
-
+import queue
 
 class Island(object):
     '''
@@ -117,6 +117,40 @@ class ClassIslandDistanceMachine():
             Labels[np.int32(x),np.int32(y)]=iIsland+1
         return Labels.reshape((1,1,nx,nx))
 
+    def BreakLargeIslands(self,ListIslands):
+        if self.GD["SSDClean"]["MaxIslandSize"]:
+            LOut=[]
+            NN=int(self.GD["SSDClean"]["MaxIslandSize"])
+            print("  breaking islands with linear size larger than %i pixels into smaller ones"%self.GD["SSDClean"]["MaxIslandSize"], file=log)
+            for iIsland,ThisIsland in enumerate(ListIslands):
+                x,y=np.array(ThisIsland).T
+                x0,x1=x.min(),x.max()
+                y0,y1=y.min(),y.max()
+                Lx,Ly=x1-x0+1,y1-y0+1
+                nx=Lx//self.GD["SSDClean"]["MaxIslandSize"]+1
+                ny=Ly//self.GD["SSDClean"]["MaxIslandSize"]+1
+                nn=nx*ny
+                if nn==1:
+                    LOut.append(ThisIsland)
+                else:
+                    print("    breaking islands #%i into %i islands"%(iIsland,nn), file=log)
+                    xb=np.int64(np.mgrid[x0:x1:(nx+1)*1j])
+                    yb=np.int64(np.mgrid[y0:y1:(ny+1)*1j])
+                    xb=xb.flatten()
+                    yb=yb.flatten()
+                    for ix in range(nx):
+                        for iy in range(ny):
+                            ind=np.where((x>xb[ix])&(x<=xb[ix+1])&(y>yb[iy])&(y<=yb[iy+1]))[0]
+                            if ind.size==0: continue
+                            II=np.zeros((ind.size,2),x.dtype)
+                            II[:,0]=x[ind]
+                            II[:,1]=y[ind]
+                            LOut.append(II)
+
+            return LOut
+        else:
+            return ListIslands
+                        
     def CalcCrossIslandPSF(self,ListIslands):
         print("  calculating global islands cross-contamination", file=log)
         PSF=np.mean(np.abs(self.PSFServer.DicoVariablePSF["MeanFacetPSF"][:,0]),axis=0)#self.PSFServer.DicoVariablePSF["MeanFacetPSF"][0,0]
@@ -313,7 +347,7 @@ class ClassIslandDistanceMachine():
             pBAR.render(NDone,NJobs)
 
     def giveEdgesIslands(self,ListIslands):
-        print("  extracting Island edges", file=log)
+        print("  extracting Island edges", file=log, end='')
         ListEdgesIslands=[]
         _,_,nx,_=self._MaskArray.shape
         #Ed=np.zeros_like(self._MaskArray)
@@ -337,7 +371,7 @@ class ClassIslandDistanceMachine():
         # pylab.imshow(Ed[0,0],interpolation="nearest")
         # pylab.draw()
         # pylab.show(False)
-
+        print("  <done>", file=log)
         return ListEdgesIslands
 
     def ConvexifyIsland(self,ListIslands):#,PolygonFile=None):
@@ -449,44 +483,51 @@ class ClassIslandDistanceMachine():
         pBAR = ProgressBar(Title="  Calc. Dist. ")
         pBAR.render(0, NJobs)
         iResult = 0
+        success=True
         if not Parallel:
             for ii in range(NCPU):
                 workerlist[ii].run()  # just run until all work is completed
-
-        while iResult < NJobs:
-            DicoResult = None
-            if result_queue.qsize() != 0:
+        try:
+            while iResult < NJobs:
+                DicoResult = None
+                
                 try:
                     DicoResult = result_queue.get()
-                except:
-                    pass
+                except queue.Empty:
+                    continue # sync call above
+                except Exception as e:
+                    print("The following unhandled exception occured.", file=log)
+                    import traceback
+                    traceback.print_tb(e.__traceback__, file=log)
+                    success = False
+                    break
 
-            if DicoResult == None:
-                time.sleep(0.5)
-                continue
+                if DicoResult is not None and DicoResult["Success"]:
+                    iResult+=1
+                    NDone=iResult
+                    pBAR.render(NDone,NJobs)
 
-            if DicoResult["Success"]:
-                iResult+=1
-                NDone=iResult
-                pBAR.render(NDone,NJobs)
-
-                iIsland=DicoResult["iIsland"]
-                Result=NpShared.GiveArray("%sDistances_%6.6i"%(self.IdSharedMem,iIsland))
-
-                self.dx[iIsland]=Result[0]
-                self.dy[iIsland]=Result[1]
-                self.D[iIsland]=Result[2]
-                NpShared.DelAll("%sDistances_%6.6i"%(self.IdSharedMem,iIsland))
-
-
-
-
-        if Parallel:
-            for ii in range(NCPU):
-                workerlist[ii].shutdown()
-                workerlist[ii].terminate()
-                workerlist[ii].join()
+                    iIsland=DicoResult["iIsland"]
+                    Result=NpShared.GiveArray("%sDistances_%6.6i"%(self.IdSharedMem,iIsland))
 
+                    self.dx[iIsland]=Result[0]
+                    self.dy[iIsland]=Result[1]
+                    self.D[iIsland]=Result[2]
+                    NpShared.DelAll("%sDistances_%6.6i"%(self.IdSharedMem,iIsland))
+        finally:
+            if Parallel:
+                for ii in range(NCPU):
+                    try:
+                        workerlist[ii].shutdown()
+                        workerlist[ii].terminate()
+                        workerlist[ii].join()
+                    except Exception as e:
+                        print("The following unhandled exception occured.", file=log)
+                        import traceback
+                        traceback.print_tb(e.__traceback__, file=log)
+        if not success:
+            raise RuntimeError("Some parallel jobs have failed. Check your log and report the issue if "
+                               "not a memory issue. Bus errors indicate memory allocation errors")
 
 
 
@@ -522,10 +563,15 @@ class WorkerDistance(multiprocessing.Process):
         Result=np.zeros((3,NIslands),np.int32)
 
         x0,y0=np.array(self.ListIslands[iIsland]).T
+
+        MaxSize=20000
+        incr0=np.max([1,int(x0.size/MaxSize)+1])
+        
         for jIsland in range(NIslands):
             x1,y1=np.array(self.ListIslands[jIsland]).T
-            dx=x0.reshape((-1,1))-x1.reshape((1,-1))
-            dy=y0.reshape((-1,1))-y1.reshape((1,-1))
+            incr1=np.max([1,int(x1.size/MaxSize)+1])
+            dx=(x0.flat[0::incr0]).reshape((-1,1))-(x1.flat[0::incr1]).reshape((1,-1))
+            dy=(y0.flat[0::incr0]).reshape((-1,1))-(y1.flat[0::incr1]).reshape((1,-1))
             d=np.sqrt(dx**2+dy**2)
             dmin=np.min(d)
             indx,indy=np.where(d==dmin)
@@ -539,20 +585,23 @@ class WorkerDistance(multiprocessing.Process):
         self.result_queue.put({"iIsland": iIsland, "Success":True})
 
     def run(self):
+        success = True
         while not self.kill_received and not self.work_queue.empty():
-            
-            DicoJob = self.work_queue.get()
-            self.giveMinDist(DicoJob)
-            # try:
-            #     self.initIsland(DicoJob)
-            # except:
-            #     iIsland=DicoJob["iIsland"]
-            #     print ModColor.Str("On island %i"%iIsland)
-            #     print traceback.format_exc()
-            #     print
-            #     print self.ListIsland[iIsland]
-            #     print
-
+            try:
+                DicoJob = self.work_queue.get()
+                self.giveMinDist(DicoJob)
+            except queue.Empty:
+                continue # sync call above
+            except Exception as e:
+                print("The following unhandled exception occured.", file=log)
+                import traceback
+                traceback.print_tb(e.__traceback__, file=log)
+                success = False
+                break            
+        if not success:
+            raise RuntimeError("Some parallel jobs have failed. Check your log and report the issue if "
+                               "not a memory issue. Bus errors indicate memory allocation errors")
+        
 
 
 
