@@ -46,6 +46,7 @@ from pyrap.images import image
 from SkyModel.Sky import ClassSM
 import os
 import copy
+from DDFacet.ToolsDir.ModToolBox import EstimateNpix
 
 class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
     def __init__(self,*args,**kwargs):
@@ -90,6 +91,29 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
 
         MyPickle.Save(D,FileName)
 
+    def ChangeNPix(self,NPixOut):
+        NPix=self.ModelShape[-1]
+        NPixOut, _ = EstimateNpix(float(NPixOut), Padding=1)
+        NPix0, _ = EstimateNpix(float(NPix), Padding=1)
+        if NPix!=NPix0: stop
+        print>>log,"Changing image size: %i -> %i pixels"%(NPix,NPixOut)
+        xc0=NPix//2
+        xc1=NPixOut//2
+        dx=xc0-xc1
+        DCompOut={}
+        DCompOut["Type"]="SSD"
+        N,M,_,_=self.ModelShape
+        self.ModelShape=[N,M,NPixOut,NPixOut]
+        for (x0,y0) in self.DicoSMStacked['Comp'].keys():
+            x1=x0-dx
+            y1=y0-dx
+            c0=(x1>=0)&(x1<NPixOut)
+            c1=(y1>=0)&(y1<NPixOut)
+            if c0&c1:
+                #print "(%i,%i)->(%i,%i)"%(x0,y0,x1,y1)
+                DCompOut[(x1,y1)]=self.DicoSMStacked['Comp'][(x0,y0)]
+        self.DicoSMStacked=DCompOut
+        
     def giveDico(self):
         D=self.DicoSMStacked
         D["GD"]=self.GD
@@ -427,8 +451,13 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
     def CleanMaskedComponants(self,MaskName,InvertMask=False):
         print("Cleaning model dictionary from masked components using %s [%i components]"%(MaskName,len(self.DicoSMStacked["Comp"])), file=log)
 
-        im=image(MaskName)
+        cwd = os.getcwd()
+        baseFitsFile = os.path.basename(MaskName)
+        dirname = os.path.dirname(os.path.abspath(MaskName))
+        os.chdir(dirname)
+        im=image(baseFitsFile)
         MaskArray=im.getdata()[0,0].T[::-1]
+        os.chdir(cwd)
         if InvertMask:
             print("  Inverting the mask", file=log)
             MaskArray=1-MaskArray
@@ -461,7 +490,12 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
             ModelMap*=(BeamImage)
 
 
-        im=image(FitsFile)
+        cwd = os.getcwd()
+        baseFitsFile = os.path.basename(FitsFile)
+        dirname = os.path.dirname(os.path.abspath(FitsFile))
+        os.chdir(dirname)
+        im=image(baseFitsFile)
+        os.chdir(cwd)
         pol,freq,decc,rac=im.toworld((0,0,0,0))
 
         Lx,Ly=np.where(ModelMap[0,0]!=0)
