@@ -45,7 +45,7 @@ def substep(A, psf, sol, Ip, Iq, pq, npixpsf):
     :param psf: the psf (not necessarily the size of the image)
     :param Ip, Iq: indices of active set relative to image
     :param pq: the location of a component relative to the active set
-    :return: 
+    :return:
     """
     halfnpixpsf = npixpsf//2
     p = Ip[pq]
@@ -121,7 +121,7 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
         else:
              self.NCPU = NCPU
         self.DoAbs = self.GD["Deconv"]["AllowNegative"]
-        self.ScaleMachine = ClassScaleMachine.ClassScaleMachine(GD=self.GD, NCPU=NCPU, MaskArray=MaskArray)
+        self.ScaleMachine = ClassScaleMachine.ClassScaleMachine(GD=self.GD, NCPU=self.NCPU, MaskArray=MaskArray)
         self.ScaleMachine.Init(PSFServer, self.FreqMachine, cachepath=cachepath, MaxBaseline=MaxBaseline)
         self.NpixPSF = self.ScaleMachine.NpixPSF
         self.halfNpixPSF = self.NpixPSF//2
@@ -171,8 +171,8 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
 
     def AppendComponentToDictStacked(self, key, Sols, iScale, Gain):
         """
-        Adds component to model dictionary at a scale specified by Scale. 
-        The dictionary corresponding to each scale is keyed on pixel values (l,m location tupple). 
+        Adds component to model dictionary at a scale specified by Scale.
+        The dictionary corresponding to each scale is keyed on pixel values (l,m location tupple).
         Each model component is therefore represented parametrically by a pixel value a scale and a set of coefficients
         describing the spectral axis.
         Currently only Stokes I is supported.
@@ -181,7 +181,7 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
             Sols: Nd array of coeffs with length equal to the number of basis functions representing the component.
             iScale: the scale index
             Gain: clean loop gain
-            
+
         Added component list to dictionary for particular scale. This dictionary is stored in
         self.DicoSMStacked["Comp"][iScale] and has keys:
             "SolsArray": solutions ndArray with shape [#basis_functions,#stokes_terms]
@@ -257,87 +257,75 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
         return ModelImage
 
     def GiveSpectralIndexMap(self, GaussPars=[(1, 1, 0)], ResidCube=None,
-                                GiveComponents=False, ChannelWeights=None):
+                             GiveComponents=False, ChannelWeights=None):
 
         # convert to radians
         ex, ey, pa = GaussPars
-        ex *= np.pi/180
-        ey *= np.pi/180
-        pa *= np.pi/180
+        ex *= np.pi/180/np.sqrt(2)/2
+        ey *= np.pi/180/np.sqrt(2)/2
+        epar = (ex + ey)/2.0
+        pa = 0.0
 
         # get in terms of number of cells
         CellSizeRad = self.GD['Image']['Cell'] * np.pi / 648000
-        # ex /= self.GD['Image']['Cell'] * np.pi / 648000
-        # ey /= self.GD['Image']['Cell'] * np.pi / 648000
 
         # get Gaussian kernel
-        GaussKern = ModFFTW.GiveGauss(self.Npix, CellSizeRad=CellSizeRad, GaussPars=(ex, ey, pa), parallel=False)
+        GaussKern = ModFFTW.GiveGauss(self.Npix, CellSizeRad=CellSizeRad, GaussPars=(epar, epar, pa), parallel=False)
 
         # take FT
         Fs = np.fft.fftshift
         iFs = np.fft.ifftshift
-        npad = self.Npad
-        FTarray = self.ScaleMachine.FTMachine.xhatim.view()
-        FTarray[...] = iFs(np.pad(GaussKern[None, None], ((0, 0), (0, 0), (npad, npad), (npad, npad)),
-                             mode='constant'), axes=(2, 3))
 
-        # this puts the FT in FTarray
-        self.ScaleMachine.FTMachine.FFTim()
+        import pyfftw
+        nthreads = int(self.GD['Parallel']['NCPU'])
+        if not nthreads:
+            import multiprocessing
+            nthreads = multiprocessing.cpu_count()
+        else:
+            from multiprocessing.pool import ThreadPool
+            import dask
 
-        # need to copy since FTarray and FTcube are views to the same array
-        FTkernel = FTarray.copy()
+            dask.config.set(pool=ThreadPool(nthreads))
+
+        FFT = lambda x: pyfftw.interfaces.numpy_fft.fft2(x, axes=(-2, -1), planner_effort='FFTW_ESTIMATE', threads=nthreads) #, norm='ortho')
+        iFFT = lambda x: pyfftw.interfaces.numpy_fft.ifft2(x, axes=(-2, -1), planner_effort='FFTW_ESTIMATE', threads=nthreads) #, norm='ortho')
 
         # evaluate model
         ModelImage = self.GiveModelImage(self.GridFreqs)
 
-        # pad and take FT
-        FTcube = self.ScaleMachine.FTMachine.Chatim.view()
-        FTcube[...] = iFs(np.pad(ModelImage, ((0, 0), (0, 0), (npad, npad), (npad, npad)),
-                                 mode='constant'), axes=(2, 3))
-        self.ScaleMachine.FTMachine.CFFTim()
+        # pad GausKern and take FT
+        GaussKern = np.pad(GaussKern, self.Npad, mode='constant')
+        FTshape, _ = GaussKern.shape
+        from scipy import fftpack as FT
+        #GaussKernhat = FT.fft2(iFs(GaussKern))
+        GaussKernhat = FFT(iFs(GaussKern))
 
-        # multiply by kernel
-        FTcube *= FTkernel
-
-        # take iFT
-        self.ScaleMachine.FTMachine.iCFFTim()
-
-        I = slice(npad, -npad)
-
-        ConvModelImage = Fs(FTcube, axes=(2,3))[:, :, I, I].real
-
-        ConvModelMean = np.mean(ConvModelImage.squeeze(), axis=0)
-
-        ConvModelLow = ConvModelImage[-1, 0]
-        ConvModelHigh = ConvModelImage[0, 0]
+        # pad and FT of ModelImage
+        ModelImagehat = np.zeros((self.Nchan, FTshape, FTshape), dtype=np.complex128)
+        ConvModelImage = np.zeros((self.Nchan, self.Npix, self.Npix), dtype=np.float64)
+        I = slice(self.Npad, -self.Npad)
+        for i in range(self.Nchan):
+            tmp_array = np.pad(ModelImage[i, 0], self.Npad, mode='constant')
+            # ModelImagehat[i] = FT.fft2(iFs(tmp_array)) * GaussKernhat
+            ModelImagehat[i] = FFT(iFs(tmp_array)) * GaussKernhat
+            # ConvModelImage[i] = Fs(FT.ifft2(ModelImagehat[i]))[I, I].real
+            ConvModelImage[i] = Fs(iFFT(ModelImagehat[i]))[I, I].real
 
         if ResidCube is not None:
-            ConvModelImage += ResidCube
-
-        ConvModelImage = ConvModelImage.squeeze()
-
-        RMS = np.std(ResidCube.flatten())
-
-        Threshold = self.GD["SPIMaps"]["AlphaThreshold"] * RMS
+            #ConvModelImage += ResidCube.squeeze()
+            RMS = np.std(ResidCube.flatten())
+            Threshold = self.GD["SPIMaps"]["AlphaThreshold"] * RMS
+        else:
+            AbsModel = np.abs(ModelImage).squeeze()
+            MinAbsImage = np.amin(AbsModel, axis=0)
+            RMS = np.min(np.abs(MinAbsImage.flatten())) # base cutoff on smallest value in model
+            Threshold = self.GD["SPIMaps"]["AlphaThreshold"] * RMS
 
         # get minimum along any freq axis
         MinImage = np.amin(ConvModelImage, axis=0)
         MaskIndices = np.argwhere(MinImage > Threshold)
         FitCube = ConvModelImage[:, MaskIndices[:, 0], MaskIndices[:, 1]]
 
-        # Initial guess for I0
-        I0i = ConvModelMean[MaskIndices[:, 0], MaskIndices[:, 1]]
-
-        # initial guess for alphas
-        Ilow = ConvModelLow[MaskIndices[:, 0], MaskIndices[:, 1]]/I0i
-        Ihigh = ConvModelHigh[MaskIndices[:, 0], MaskIndices[:, 1]]/I0i
-        alphai = (np.log(Ihigh) - np.log(Ilow))/(np.log(self.GridFreqs[0]/self.RefFreq) - np.log(self.GridFreqs[-1]/self.RefFreq))
-
-        # import matplotlib.pyplot as plt
-        #
-        # for i in range(self.Nchan):
-        #     plt.imshow(np.where(ConvModelImage[i] > Threshold, ConvModelImage[i], 0.0))
-        #     plt.show()
 
         if ChannelWeights is None:
             weights = np.ones(self.Nchan, dtype=np.float32)
@@ -351,38 +339,17 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
         try:
             import traceback
             from africanus.model.spi.dask import fit_spi_components
-            NCPU = self.GD["Parallel"]["NCPU"]
-            if NCPU:
-                from multiprocessing.pool import ThreadPool
-                import dask
-
-                dask.config.set(pool=ThreadPool(NCPU))
-            else:
-                import multiprocessing
-                NCPU = multiprocessing.cpu_count()
-
             import dask.array as da
             _, ncomps = FitCube.shape
-            FitCubeDask = da.from_array(FitCube.T.astype(np.float64), chunks=(ncomps//NCPU, self.Nchan))
+            FitCubeDask = da.from_array(FitCube.T.astype(np.float64),
+                                        chunks=(np.maximum(100, ncomps//nthreads), self.Nchan))
             weightsDask = da.from_array(weights.astype(np.float64), chunks=(self.Nchan))
-            freqsDask = da.from_array(self.GridFreqs.astype(np.float64), chunks=(self.Nchan))
+            freqsDask = da.from_array(np.array(self.GridFreqs).astype(np.float64), chunks=(self.Nchan))
 
             alpha, varalpha, Iref, varIref = fit_spi_components(FitCubeDask, weightsDask,
-                                                                freqsDask, self.RefFreq,
-                                                                dtype=np.float64, I0i=I0i,
-                                                                alphai=alphai).compute()
-
-            # from africanus.model.spi import fit_spi_components
-            #
-            # alpha, varalpha, Iref, varIref = fit_spi_components(FitCube.T.astype(np.float64), weights.astype(np.float64),
-            #                                                     self.GridFreqs.astype(np.float64), self.RefFreq.astype(np.float64),
-            #                                                     dtype=np.float64, I0i=I0i, alphai=alphai)
+                                                                freqsDask, self.RefFreq).compute()
         except Exception as e:
-            traceback_str = traceback.format_exc(e)
-            print("Warning - Failed at importing africanus spi fitter. This could be an issue with the dask " \
-                        "version. Falling back to (slow) scipy version", file=log)
-            print("Original traceback - ", traceback_str, file=log)
-            alpha, varalpha, Iref, varIref = self.FreqMachine.FitSPIComponents(FitCube, self.GridFreqs, self.RefFreq)
+            raise(e)
 
         _, _, nx, ny = ModelImage.shape
         alphamap = np.zeros([nx, ny])
@@ -454,19 +421,19 @@ class ClassModelMachine(ClassModelMachinebase.ClassModelMachine):
 
     def do_minor_loop(self, Dirty, meanDirty, JonesNorm, WeightsChansImages, MaxDirty, Stopping_flux=None, RMS=None):
         """
-        Runs the sub-minor loop at a specific scale 
+        Runs the sub-minor loop at a specific scale
         :param Dirty: dirty cube
         :param meanDirty: mean dirty
         :param JonesNorm: "average" DDE map
         :param WeightsChansImages: The sum of the weights in each channel normalised to sum to one
         :param MaxDirty: maximum of mean dirty computed in last minor loop
         :param Stopping_flux: stopping flux for unconvolved mean image
-        :param RMS: RMS of mean dirty computed in last minor loop (for auto-masking) 
+        :param RMS: RMS of mean dirty computed in last minor loop (for auto-masking)
         :return: number of iterations k, the dominant scale
-        
+
         Components are only searched for in the active set A defined as all pixels in s scale convolved dirty
         above GD[WSCMS][SubMinorPeakFactor] * AbsConvMaxDirty. The PSF for A is the PSF twice convolved with the
-        dominant scale kernel. At the same time, once a component has been found in A, it is subtracted from the 
+        dominant scale kernel. At the same time, once a component has been found in A, it is subtracted from the
         dirty cube using the PSF once convolved with the scale kernel. The actual MeanDirty image is only updated
         once we drop back into the minor loop by computing the weighted sum over channels.
         """
