@@ -23,23 +23,39 @@ from DDFacet.ToolsDir import ModCoord
 class ClassSM():
     def __init__(self,infile,infile_cluster="",killdirs=[],invert=False,DoPrintCat=False,\
                      ReName=False,DoREG=False,SaveNp=False,NCluster=0,DoPlot=True,Tigger=False,\
-                     FromExt=None,ClusterMethod=1,SelSource=False):
+                     FromExt=None,ClusterMethod=1,SelSource=False,DoPrint=True):
         self.ClusterMethod=ClusterMethod
         self.infile_cluster=infile_cluster
         self.TargetList=infile
         self.Type="Catalog"
-
-        self.DoPrint=True
-        if type(infile).__name__=="instance":
-            Cat=infile.SourceCat
+        self.DoPrint=DoPrint
+        if (type(infile).__name__=="instance") or (type(infile).__name__=="ClassImageSM") or (type(infile).__name__=="ClassSM"):
+            Cat=infile.SourceCat.copy()
             Cat=Cat.view(np.recarray)
             self.DoPrint=0
+# =======
+#         if "instance" in str(type(infile)):
+#             ClusterCat=infile.ClusterCat.copy()
+#             NN=ClusterCat.shape[0]
+#             Cat=np.zeros((NN,),dtype=[('Name','|S200'),('ra',np.float),('dec',np.float),('Sref',np.float),('I',np.float),('Q',np.float),\
+#                                                        ('U',np.float),('V',np.float),('RefFreq',np.float),('alpha',np.float),('ESref',np.float),\
+#                                                        ('Ealpha',np.float),('kill',np.int),('Cluster',np.int),('Type',np.int),('Gmin',np.float),\
+#                                                        ('Gmaj',np.float),('Gangle',np.float),("Select",np.int),('l',np.float),('m',np.float),
+#                                                        ("Exclude",bool)])
+#             Cat=Cat.view(np.recarray)
+#             Cat.RefFreq=1.
+#             Cat.ra[:]=ClusterCat.ra
+#             Cat.dec[:]=ClusterCat.dec
+#             Cat.I[:]=ClusterCat.SumI[:]
+#             Cat.Cluster=np.arange(NN)
+#             Cat.Sref[:]=ClusterCat.SumI[:]
+# >>>>>>> TestASKAP
         elif ".npy" in infile:
             Cat=np.load(infile)
             Cat=Cat.view(np.recarray)
         elif Tigger:
             Cat=ModTigger.ReadTiggerModel(infile)
-        elif FromExt!=None:
+        elif FromExt is not None:
             
             Cat=ModSMFromNp.ReadFromNp(FromExt)
         else:
@@ -88,18 +104,26 @@ class ClassSM():
             for i in range(len(self.SourceCat)):
                 for StrPiece in killdirs:
                     if StrPiece=="": continue
-                    if StrPiece in self.SourceCat.Name[i]: self.SourceCat.kill[i]=1
+                    Name=self.SourceCat.Name[i]
+                    if "byte" in type(Name).__name__: Name=Name.decode("utf-8")
+                    if StrPiece in Name: self.SourceCat.kill[i]=1
+                    
         if invert:
             ind0=np.where(self.SourceCat.kill==0)[0]
             ind1=np.where(self.SourceCat.kill==1)[0]
             self.SourceCat.kill[ind0]=1
             self.SourceCat.kill[ind1]=0
 
+        # print(self.SourceCat.Name)
+        # print(self.SourceCat.kill)
+            
+        #log.print(ModColor.Str())
+
     def Save(self):
         infile=self.infile
         print(ModColor.Str(" SkyModel PROPERTIES: "))
         print("   - SkyModel File Name: %s"%ModColor.Str(infile,col="green"))
-        if self.REGFile!=None: print("   - ds9 region file: %s"%ModColor.Str(self.REGFile,col="green"))
+        if self.REGFile is not None: print("   - ds9 region file: %s"%ModColor.Str(self.REGFile,col="green"))
         npext=""
         if not(".npy" in infile): npext=".npy"
         self.NpFile="%s%s"%(infile,npext)
@@ -113,6 +137,7 @@ class ClassSM():
 
     def PrintBasics(self):
         infile=self.infile
+        # if "instance" in str(type(infile)): return
         npext=""
         if not(".npy" in infile): npext=".npy"
         print("   - Numpy catalog file: %s"%ModColor.Str("%s%s"%(infile,npext),col="green"))
@@ -122,6 +147,9 @@ class ClassSM():
         #print "   - Cluster File Name: %s"%self.infile_cluster
         print("   - Number of Sources  = ",self.SourceCat.shape[0])
         print("   - Number of Directions  = ",self.NDir)
+        Np=np.count_nonzero(self.SourceCat.Type==0)
+        Ng=np.count_nonzero(self.SourceCat.Type==1)
+        print("   - Number of [ POINT | GAUSSIANS ] : [ %i | %i ]"%(Np,Ng))
         print()
 
     def Cluster(self,NCluster=1,DoPlot=True,PreCluster="",FromClusterCat=""):
@@ -137,7 +165,7 @@ class ClassSM():
             PreClusterCat=None
             ExcludeCat=None
 
-        if ExcludeCat!=None:
+        if ExcludeCat is not None:
             for j in range(ExcludeCat.shape[0]):
                 d=np.sqrt((self.SourceCat.ra-ExcludeCat.ra[j])**2+(self.SourceCat.dec-ExcludeCat.dec[j])**2)
                 self.SourceCat.Exclude[d<ExcludeCat.Radius[j]]=True
@@ -205,7 +233,7 @@ class ClassSM():
         NPreCluster=0
 
         # #######################################
-        # if (PreClusterCat!=None)&(FromClusterCat==""):
+        # if (PreClusterCat is not None)&(FromClusterCat==""):
         #     N=PreClusterCat.shape[0]
         #     Ns=self.SourceCat.ra.shape[0]
         #     for iReg in range(N):
@@ -254,7 +282,7 @@ class ClassSM():
         elif self.ClusterMethod==3:
             CM=ClassClusterRadial(x,y,s,nk,DoPlot=DoPlot)
         elif self.ClusterMethod==4:
-            if PreClusterCat!=None:
+            if PreClusterCat is not None:
                 l0,m0=self.radec2lm_scalar(PreClusterCat.ra,PreClusterCat.dec)
                 CM=ClassClusterKMean(x,y,s,nk,DoPlot=DoPlot,PreCluster=(l0,m0))
             else:
@@ -318,7 +346,7 @@ class ClassSM():
                 DictNode["%3.3i"%iCluster]["ListCluster"]=ind.tolist()
                 # pylab.scatter(x[ind],y[ind],c=np.ones((ind.size,))*iCluster,vmin=0,vmax=Nc,lw=0)
                 # pylab.draw()
-                # pylab.show(False)
+                # pylab.show(block=False)
             
 
 
@@ -342,7 +370,7 @@ class ClassSM():
 
 
 
-        # if PreClusterCat!=None:
+        # if PreClusterCat is not None:
         #     SourceCat=np.concatenate((SourceCatPreCluster,SourceCat))
         #     SourceCat=SourceCat.view(np.recarray)
 
@@ -441,16 +469,17 @@ class ClassSM():
     def Calc_LM(self,rac,decc):
         Cat=self.SourceCat
         if not("l" in list(Cat.dtype.fields.keys())):
-            Cat=RecArrayOps.AppendField(Cat,('l',float))
-            Cat=RecArrayOps.AppendField(Cat,('m',float))
+            Cat=RecArrayOps.AppendField(Cat,'l',float)
+            Cat=RecArrayOps.AppendField(Cat,'m',float)
+
         Cat.l,Cat.m=self.radec2lm_scalar(self.SourceCat.ra,self.SourceCat.dec,rac,decc)
         self.SourceCat=Cat
         self.SourceCatKeepForSelector=self.SourceCat.copy()
 
         Cat=self.ClusterCat
         if not("l" in list(Cat.dtype.fields.keys())):
-            Cat=RecArrayOps.AppendField(Cat,('l',float))
-            Cat=RecArrayOps.AppendField(Cat,('m',float))
+            Cat=RecArrayOps.AppendField(Cat,'l',float)
+            Cat=RecArrayOps.AppendField(Cat,'m',float)
         Cat.l,Cat.m=self.radec2lm_scalar(self.ClusterCat.ra,self.ClusterCat.dec,rac,decc)
         self.ClusterCat=Cat
     # def Calc_LM(self,rac,decc):
